<?php

/*
Plugin Name: Events Optimizer
Plugin URI: https://www.heinrich.biz
Description: Use AI to improve your event listings with accurate and up-to-date information, enhancing your SEO and captivating your audience. Eliminate the hassle of managing incorrect or outdated details by automating the optimization of data about events, venues, and organizers with just a few clicks. Requires The Events Calendar Plugin.
Version: 1.0.16
Author: Oliver Heinrich
Text Domain: events-optimizer
Domain Path: /languages
*/
//----------------------------------------------------------------------------------------
// Load Freemius SDK
// https://www.freemius.com
//----------------------------------------------------------------------------------------

if ( !function_exists( 'teo_fs' ) ) {
    // Create a helper function for easy SDK access.
    function teo_fs()
    {
        global  $teo_fs ;
        
        if ( !isset( $teo_fs ) ) {
            // Include Freemius SDK.
            require_once dirname( __FILE__ ) . '/vendor/freemius/start.php';
            $teo_fs = fs_dynamic_init( array(
                'id'             => '8391',
                'slug'           => 'events-optimizer',
                'premium_slug'   => 'events-optimizer-pro',
                'type'           => 'plugin',
                'public_key'     => 'pk_94f588954e6ff1b7a22a3734cae8a',
                'is_premium'     => false,
                'has_addons'     => false,
                'has_paid_plans' => true,
                'trial'          => array(
                'days'               => 3,
                'is_require_payment' => true,
            ),
                'menu'           => array(
                'slug'           => 'events_optimizer',
                'override_exact' => true,
                'first-path'     => 'edit.php?post_type=tribe_events&page=events_optimizer&tab=tab-4',
                'support'        => false,
                'parent'         => array(
                'slug' => 'edit.php?post_type=tribe_events',
            ),
            ),
                'is_live'        => true,
            ) );
        }
        
        return $teo_fs;
    }
    
    // Init Freemius.
    teo_fs();
    // Signal that SDK was initiated.
    do_action( 'teo_fs_loaded' );
    function teo_fs_settings_url()
    {
        return admin_url( 'edit.php?post_type=tribe_events&page=events_optimizer&tab=tab-4' );
    }
    
    teo_fs()->add_filter( 'connect_url', 'teo_fs_settings_url' );
    teo_fs()->add_filter( 'after_skip_url', 'teo_fs_settings_url' );
    teo_fs()->add_filter( 'after_connect_url', 'teo_fs_settings_url' );
    teo_fs()->add_filter( 'after_pending_connect_url', 'teo_fs_settings_url' );
}

//----------------------------------------------------------------------------------------
// Load translations
//----------------------------------------------------------------------------------------
add_action( 'plugins_loaded', 'teo_load_plugin_textdomain' );
function teo_load_plugin_textdomain()
{
    load_plugin_textdomain( 'events-optimizer', false, basename( dirname( __FILE__ ) ) . '/languages/' );
}

//----------------------------------------------------------------------------------------
// Initialize menu and settings
//----------------------------------------------------------------------------------------
// Add Optimizer to Events Calendar menu in WordPress dashboard
add_action( 'admin_menu', 'teo_initialize_menu' );
function teo_initialize_menu()
{
    // Add Menu to Events Calendar
    $my_page = add_submenu_page(
        'edit.php?post_type=tribe_events',
        'Events Optimizer',
        'Optimizer',
        'administrator',
        'events_optimizer',
        'events_optimizer_menu'
    );
    // Call register settings function
    add_action( 'admin_init', 'teo_register_settings' );
    // Load the JS conditionally (see next function below)
    add_action( 'load-' . $my_page, 'teo_load_admin_js' );
}

//----------------------------------------------------------------------------------------
// This function is only called when the Event Optimizers page is loaded
// https://wordpress.stackexchange.com/questions/41207/how-do-i-enqueue-styles-scripts-on-certain-wp-admin-pages
//----------------------------------------------------------------------------------------
function teo_load_admin_js()
{
    // Register against the proper action hook
    add_action( 'admin_enqueue_scripts', 'teo_enqueue_admin_js' );
}

//----------------------------------------------------------------------------------------
// Include CSS and JavaScript files
//----------------------------------------------------------------------------------------
function teo_enqueue_admin_js()
{
    // Load the scripts that have been registered above
    wp_enqueue_script( 'tabs.js', plugin_dir_url( __FILE__ ) . 'assets/js/tabs.js' );
    wp_enqueue_style( 'style.css', plugin_dir_url( __FILE__ ) . 'assets/css/style.css' );
}

//----------------------------------------------------------------------------------------
// Register settings
//----------------------------------------------------------------------------------------
function teo_register_settings()
{
    register_setting( 'events-optimizer-settings', 'teo_max_results_per_page' );
    //register_setting( 'events-optimizer-settings', 'teo_remove_settings' );
    // Set default values
    if ( !get_option( 'teo_max_results_per_page' ) ) {
        update_option( 'teo_max_results_per_page', 5 );
    }
    // endif teo_fs()->can_use_premium_code__premium_only()
}

//----------------------------------------------------------------------------------------
// Messages to show in WordPress Dashboard
//----------------------------------------------------------------------------------------
if ( get_option( 'teo_update_notice' ) != true ) {
    add_action( 'admin_notices', 'teo_admin_notice' );
}
function teo_admin_notice()
{
    // Check if The Events Calendar plugin is installed and active
    
    if ( is_plugin_active( 'the-events-calendar/the-events-calendar.php' ) ) {
        // Show infos about the latest update
        ?><div class="notice notice-success teo-update-notice is-dismissible">
			<p>
			<?php 
        \esc_html_e( 'With this version you can create descriptions for events, venues and organizers automatically with AI. Check out the', 'events-optimizer' );
        ?> 
			<a href="/wp-admin/edit.php?post_type=tribe_events&page=events_optimizer&tab=tab-4">
			<?php 
        \esc_html_e( 'Events Optimizer settings.', 'events-optimizer' );
        ?>
			</a>
			</p>			
		</div><?php 
    }

}

if ( get_option( 'teo_tec_installed_notice' ) != true ) {
    add_action( 'admin_notices', 'tec_admin_tec_installed_notice' );
}
function tec_admin_tec_installed_notice()
{
    // Check if The Events Calendar plugin is installed and active
    
    if ( is_plugin_active( 'the-events-calendar/the-events-calendar.php' ) ) {
        ?><div class="notice notice-success teo-tec-installed-notice is-dismissible">
			<p>
			<?php 
        \esc_html_e( 'Events Optimizer installation successful. Before using it please create a backup of your website and be sure to review the ', 'events-optimizer' );
        ?>
			<a href="/wp-admin/edit.php?post_type=tribe_events&page=events_optimizer&tab=tab-4">
			<?php 
        \esc_html_e( 'Events Optimizer settings.', 'events-optimizer' );
        ?>
			</a>
			</p>			
		</div><?php 
    } else {
        ?><div class="notice notice-error teo-tec-installed-notice">
			<p>
			<?php 
        \esc_html_e( 'The Events Calendar Plugin is not activated! Please install and activate it before using the Events Optimizer!', 'events-optimizer' );
        ?>
			</p>			
		</div><?php 
    }

}

//----------------------------------------------------------------------------------------
// Enque script for admin notices
//----------------------------------------------------------------------------------------
add_action( 'admin_enqueue_scripts', 'add_script' );
function add_script()
{
    wp_register_script(
        'notice-update',
        plugin_dir_url( __FILE__ ) . 'assets/js/notice-update.js',
        '',
        '1.0',
        false
    );
    wp_localize_script( 'notice-update', 'notice_params', array(
        'ajaxurl' => get_admin_url() . 'admin-ajax.php',
    ) );
    wp_enqueue_script( 'notice-update' );
}

//----------------------------------------------------------------------------------------
// If notices are dismissed don't show it again
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_my_dismiss_notice', 'my_dismiss_notice' );
function my_dismiss_notice()
{
    update_option( 'teo_update_notice', true );
}

add_action( 'wp_ajax_my_dismiss_notice_tec_installed', 'my_dismiss_notice_tec_installed' );
function my_dismiss_notice_tec_installed()
{
    update_option( 'teo_tec_installed_notice', true );
}

//----------------------------------------------------------------------------------------
// Main function: Create Menu / Tabs
//----------------------------------------------------------------------------------------
function events_optimizer_menu()
{
    // Check if user has admin permissions
    if ( !current_user_can( 'manage_options' ) ) {
        wp_die( __( 'You are not authorized to view this page!', 'events-optimizer' ) );
    }
    // Get settings
    $max_results_per_page = get_option( 'teo_max_results_per_page' );
    $phone_number_format = get_option( 'teo_phone_number_format' );
    $address_format = get_option( 'teo_address_format' );
    $preselection = get_option( 'teo_preselection' );
    $nonce = wp_create_nonce( 'events-optimizer-nonce' );
    // Create submenu
    ?>
	<div class='wrap'>	
		<h1 class="wp-heading-inline"><?php 
    esc_html_e( 'Events Optimizer', 'events-optimizer' );
    ?></h1>
	
		<?php 
    $active_tab = $_GET['tab'] ?? '';
    if ( !in_array( $active_tab, [
        'tab-1',
        'tab-2',
        'tab-3',
        'tab-4'
    ] ) ) {
        $active_tab = 'tab-1';
    }
    ?>

		<!-- Setup Tabs -->
		<!-- // https://epiph.yt/blog/2018/einfache-javascript-tabs-im-wordpress-backend/ -->
		<h2 class="nav-tab-wrapper">
			<a href="edit.php?post_type=tribe_events&page=events_optimizer&tab=tab-1" class="nav-tab<?php 
    echo  ( $active_tab === 'tab-1' || empty($active_tab) ? ' nav-tab-active' : '' ) ;
    ?>" data-slug= 'tab-1' ><?php 
    \esc_html_e( 'Venues', 'events-optimizer' );
    ?></a>
			<a href="edit.php?post_type=tribe_events&page=events_optimizer&tab=tab-2" class="nav-tab<?php 
    echo  ( $active_tab === 'tab-2' ? ' nav-tab-active' : '' ) ;
    ?>" data-slug= 'tab-2' ><?php 
    \esc_html_e( 'Organizers', 'events-optimizer' );
    ?></a>
			<a href="edit.php?post_type=tribe_events&page=events_optimizer&tab=tab-3" class="nav-tab<?php 
    echo  ( $active_tab === 'tab-3' ? ' nav-tab-active' : '' ) ;
    ?>" data-slug= 'tab-3' ><?php 
    \esc_html_e( 'Events', 'events-optimizer' );
    ?></a>
			<a href="edit.php?post_type=tribe_events&page=events_optimizer&tab=tab-4" class="nav-tab<?php 
    echo  ( $active_tab === 'tab-4' ? ' nav-tab-active' : '' ) ;
    ?>" data-slug= 'tab-4' ><?php 
    \esc_html_e( 'Settings', 'events-optimizer' );
    ?></a>
		</h2>
		

		<!-- Add Venues content to tabs -->
		<div class="nav-tab-content<?php 
    echo  ( $active_tab === 'tab-1' || empty($active_tab) ? ' nav-tab-content-active' : '' ) ;
    ?>" id="nav-tab-content-tab-1">
			<p><?php 
    \esc_html_e( 'Select the data you want to analyze.', 'events-optimizer' );
    ?></p>

			<div class='wrap'>
				<form action='' method='POST' id='venues_form'>				
					<input type='button' name='action' class='button' id='venues_with_invalid_data' value='<?php 
    esc_html_e( 'Venues with invalid data', 'events-optimizer' );
    ?>' />&nbsp;&nbsp;&nbsp;
					<input type='button' name='action' class='button' id='duplicate_venues' value='<?php 
    esc_html_e( 'Duplicate venues', 'events-optimizer' );
    ?>' <?php 
    echo  "disabled" ;
    ?>  />&nbsp;&nbsp;&nbsp; 
					<input type='button' name='action' class='button' id='venues_google_places_optimization' value='<?php 
    esc_html_e( 'Optimization with Google Places', 'events-optimizer' );
    ?>' <?php 
    echo  "disabled" ;
    ?>/>&nbsp;&nbsp;&nbsp;  
				</form>
				<p>
				<div id='show_results_venues'></div>
				<div id='loading_tab_venues'><?php 
    \esc_attr_e( 'ANALYZING', 'events-optimizer' );
    ?></div>
				</p>
			</div>		
		</div>
		

		<!-- Add Organizers content to tabs -->
		<div class="nav-tab-content<?php 
    echo  ( $active_tab === 'tab-2' ? ' nav-tab-content-active' : '' ) ;
    ?>" id="nav-tab-content-tab-2">
			<p><?php 
    \esc_html_e( 'Select the data you want to analyze.', 'events-optimizer' );
    ?></p>

			<div class='wrap'>
				<form action='' method='POST' id='organizers_form'>
					<input type='button' name='action' class='button' id='organizers_with_invalid_data' value='<?php 
    esc_html_e( 'Organizers with invalid data', 'events-optimizer' );
    ?>' />&nbsp;&nbsp;&nbsp;
					<input type='button' name='action' class='button' id='duplicate_organizers' value='<?php 
    esc_html_e( 'Duplicate organizers', 'events-optimizer' );
    ?>' <?php 
    echo  "disabled" ;
    ?>  />&nbsp;&nbsp;&nbsp; 
					<input type='button' name='action' class='button' id='organizers_google_places_optimization' value='<?php 
    esc_html_e( 'Optimization with Google Places', 'events-optimizer' );
    ?>' <?php 
    echo  "disabled" ;
    ?> />&nbsp;&nbsp;&nbsp;  
				</form>
				<p>
				<div id='show_results_organizers'></div>
				<div id='loading_tab_organizers'><?php 
    \esc_attr_e( 'ANALYZING', 'events-optimizer' );
    ?></div>
				</p>
			</div>			
		</div>
		
		
		<!-- Add Events content to tabs -->
		<div class="nav-tab-content<?php 
    echo  ( $active_tab === 'tab-3' ? ' nav-tab-content-active' : '' ) ;
    ?>" id="nav-tab-content-tab-3">
			<p><?php 
    \esc_html_e( 'Select the data you want to analyze or add. Be aware that adding events is a 2-step approach.', 'events-optimizer' );
    ?></p>

			<div class='wrap'>
				<form action='' method='POST' id='events_form'>
					<span class='frame_around'>
						<input type='button' name='action' class='button' id='events_with_invalid_data' value='<?php 
    esc_html_e( 'Events with invalid data', 'events-optimizer' );
    ?>'/>
					</span>
					<span class='frame_around'>
						<?php 
    \esc_html_e( 'Keyword', 'events-optimizer' );
    ?> <input type="text" name="dataforseo_keyword" id="dataforseo_keyword" size="14" value="<?php 
    echo  esc_attr( get_option( 'teo_dataforseo_keyword' ) ) ;
    ?>"<?php 
    echo  "disabled" ;
    ?> />&nbsp;&nbsp;&nbsp;
						<?php 
    \esc_html_e( 'Max. results', 'events-optimizer' );
    ?> <input type="text" name="dataforseo_depth" id="dataforseo_depth" size="2" value="<?php 
    echo  esc_attr( get_option( 'teo_dataforseo_depth' ) ) ;
    ?>"  <?php 
    echo  "disabled" ;
    ?> />&nbsp;&nbsp;
						<input type='button' name='action' class='button' id='search_events_with_google_data' value='<?php 
    esc_html_e( 'Search for events', 'events-optimizer' );
    ?>' <?php 
    echo  "disabled" ;
    ?>/>&nbsp;&nbsp;&nbsp;&#10132;&nbsp;&nbsp;&nbsp;<input type='button' name='action' class='button' id='add_events_with_google_data' value='<?php 
    esc_html_e( 'Show results', 'events-optimizer' );
    ?>' <?php 
    echo  "disabled" ;
    ?>/>
					</span>
				</form>
				<p>
				<div id='show_results_events'></div>
				<div id='loading_tab_events'><?php 
    \esc_attr_e( 'ANALYZING', 'events-optimizer' );
    ?></div>
				</p>
			</div>			
		</div>	

			
		<!-- Add Settings content to tabs -->
		<div class="nav-tab-content<?php 
    echo  ( $active_tab === 'tab-4' ? ' nav-tab-content-active' : '' ) ;
    ?>" id="nav-tab-content-tab-4">
			<div>
			
				<form action='' method='POST' id='settings_form'>

					<div class="tribe-settings-form-wrap">
						<br><h3><?php 
    \esc_html_e( 'General', 'events-optimizer' );
    ?></h3>
						<?php 
    ?>
						<div id="modern-tribe-info"><?php 
    \esc_html_e( 'Upgrade to Events Optimizer Pro! Enjoy amazing features like AI boosted generating descriptions for events, venues and organizers, finding duplicate venues or organizers and auto completion of venue and organizer data via Google Places. And change all settings that are greyed out! Get more information at ', 'events-optimizer' );
    ?>
							<a href="https://www.heinrich.biz" target="_blank">
							<?php 
    \esc_html_e( 'our website!', 'events-optimizer' );
    ?>
							</a>
						</div>
						<?php 
    ?>						
					
						<table class="form-table">
							<?php 
    ?>
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Events Optimizer Pro Licence Key', 'events-optimizer' );
    ?></th>
							<td><a href="https://checkout.freemius.com/mode/dialog/plugin/8391/plan/13936/" class="button" target="_blank"><?php 
    \esc_attr_e( 'Get licence key', 'events-optimizer' );
    ?></a></td>
							</tr>
							<?php 
    ?>						
		
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Maximum results per page', 'events-optimizer' );
    ?></th>
							<td><input type="text" name="max_results_per_page" id="max_results_per_page" size="5" value="<?php 
    echo  esc_attr( get_option( 'teo_max_results_per_page' ) ) ;
    ?>"/>
								<?php 
    \esc_html_e( '(max = 20 to avoid time-outs)', 'events-optimizer' );
    ?></td>
							</tr>
							<!--
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Remove settings during uninstall?', 'events-optimizer' );
    ?> </th>
							<td><input type="checkbox" name="remove_settings" id="remove_settings" value="1" <?php 
    checked( get_option( 'teo_remove_settings' ), 1 );
    ?>/></td>
							</tr>
							-->
						</table>
			
						<br><h3><?php 
    \esc_html_e( 'Duplicate venues / organizers', 'events-optimizer' );
    ?></h3>				
						<div id="modern-tribe-info">
							<?php 
    \esc_html_e( 'With the compare value you can control how identical objects are found. The smaller the value the more identical both objects need to be.', 'events-optimizer' );
    ?><br>
							<li><?php 
    \esc_html_e( 'For venues the name, the address and the first three characters of the ZIP will be used to find identical venues.', 'events-optimizer' );
    ?></li>
							<li><?php 
    \esc_html_e( 'For organizers only the name will be used to find identical organizers.', 'events-optimizer' );
    ?></li>				
						</div>
						<table class="form-table">	
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Duplicate organizers compare value', 'events-optimizer' );
    ?></th>
							<td><input type="text" name="duplicate_organizers_levenshtein" id="duplicate_organizers_levenshtein" size="5" value="<?php 
    echo  esc_attr( get_option( 'teo_duplicate_organizers_levenshtein' ) ) ;
    ?>"  <?php 
    echo  "disabled" ;
    ?> />
								<?php 
    \esc_html_e( '(0 = exact match, recommended = 0.15 - 0.3)', 'events-optimizer' );
    ?>
							</td>
							</tr>
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Duplicate venues compare value', 'events-optimizer' );
    ?></th>
							<td><input type="text" name="duplicate_venues_levenshtein" id="duplicate_venues_levenshtein" size="5" value="<?php 
    echo  esc_attr( get_option( 'teo_duplicate_venues_levenshtein' ) ) ;
    ?>"  <?php 
    echo  "disabled" ;
    ?> />
								<?php 
    \esc_html_e( '(0 = exact match, recommended = 0.35)', 'events-optimizer' );
    ?>
							</td>
							</tr>
						</table>

						<br><h3><?php 
    \esc_html_e( 'Generate descriptions for events, venues and organizers', 'events-optimizer' );
    ?><span class='beta_version'>BETA</span></h3>
						<div id="modern-tribe-info">

							<?php 
    \esc_html_e( 'To automatically generate description texts the OpenAI API is used.', 'events-optimizer' );
    ?><br>
							<li><strong><?php 
    \esc_html_e( 'OpenAI can generate false output! Please ensure to double check the output before publishing it!', 'events-optimizer' );
    ?></strong></li>
							<li><?php 
    \esc_html_e( 'To set up and configure your API key check out this', 'events-optimizer' );
    ?> <a href="https://elephas.app/blog/how-to-create-openai-api-keys-cl5c4f21d281431po7k8fgyol0" target="_blank">
								<?php 
    \esc_html_e( 'manual.', 'events-optimizer' );
    ?></a>
								<?php 
    \esc_html_e( 'More information can be found on the', 'events-optimizer' );
    ?> <a href="https://platform.openai.com/account/api-keys" target="_blank">
								<?php 
    \esc_html_e( 'OpenAI website.', 'events-optimizer' );
    ?></a>
							</li>										
							<li><?php 
    \esc_html_e( 'Be aware that the OpenAI language model has been training only with data up to 2021, which can result in outdated output!', 'events-optimizer' );
    ?></li>
							<li><?php 
    \esc_html_e( 'Sometimes OpenAI is not available. You can check the', 'events-optimizer' );
    ?> <a href="https://status.openai.com/" target="_blank">
								<?php 
    \esc_html_e( 'status here.', 'events-optimizer' );
    ?></a>
							</li>	
							<li><?php 
    \esc_html_e( 'Before using it please review the', 'events-optimizer' );
    ?> <a href="https://openai.com/api/pricing/" target="_blank">
							<?php 
    \esc_html_e( 'pricing of the OpenAI API.', 'events-optimizer' );
    ?></a></li>					
									
						</div>
															
						<table class="form-table">
							<tr valign="top">					
							<th scope="row"><?php 
    \esc_html_e( 'OpenAI API key', 'events-optimizer' );
    ?></th>
							<td><input type="text" name="openai_api_key" id="openai_api_key" size="60"  value="<?php 
    echo  esc_attr( get_option( 'teo_openai_api_key' ) ) ;
    ?>"  <?php 
    echo  "disabled" ;
    ?> />
							</td>
							</tr>					
							<tr valign="top">					
							<th scope="row"><?php 
    \esc_html_e( 'Model', 'events-optimizer' );
    ?></th>
							<td>
							<select name="openai_model" id="openai_model" <?php 
    echo  "disabled" ;
    ?> >  
							<?php 
    $selected_openai_model = get_option( 'teo_openai_model' );
    $model_options = array(
        'text-davinci-002' => __( 'Davinci 002', 'events-optimizer' ),
        'text-davinci-003' => __( 'Davinci 003', 'events-optimizer' ),
        'gpt-3.5-turbo'    => __( 'GPT-3.5 turbo', 'events-optimizer' ),
        'gpt-4'            => __( 'GPT-4', 'events-optimizer' ),
    );
    // Outdated / not working models:
    // 'text-davinci-001'		=> __( 'Davinci 001', 'events-optimizer' ),
    //	'gpt-4-32k'				=> __( 'GPT-4 32K', 'events-optimizer' )
    foreach ( $model_options as $model_value => $model_decription ) {
        
        if ( $model_value === $selected_openai_model ) {
            echo  '<option value="' . $model_value . '" selected>' . $model_decription . '</option>' ;
        } else {
            echo  '<option value="' . $model_value . '">' . $model_decription . '</option>' ;
        }
    
    }
    ?>
							</select>
							<?php 
    \esc_html_e( '(Find more details about the available models in the', 'events-optimizer' );
    ?>
							<a href="https://platform.openai.com/docs/models/overview" target="_blank">
							<?php 
    \esc_html_e( 'documentation.', 'events-optimizer' );
    ?></a>
							</td>
							</tr>
							<tr valign="top">					
							<th scope="row"><?php 
    \esc_html_e( 'Language', 'events-optimizer' );
    ?></th>
							<td><input type="text" name="openai_language" id="openai_language" size="10"  value="<?php 
    echo  esc_attr( get_option( 'teo_openai_language' ) ) ;
    ?>"  <?php 
    echo  "disabled" ;
    ?> />
							</td>
							</tr>								
							<tr valign="top">					
							<th scope="row"><?php 
    \esc_html_e( 'Max tokens', 'events-optimizer' );
    ?></th>
							<td><input type="text" name="openai_max_tokens" id="openai_max_tokens" size="5"  value="<?php 
    echo  esc_attr( get_option( 'teo_openai_max_tokens' ) ) ;
    ?>"  <?php 
    echo  "disabled" ;
    ?> />
							<?php 
    \esc_html_e( '(2048 tokens are about 900 words. Calculate them ', 'events-optimizer' );
    ?>
							<a href="https://platform.openai.com/tokenizer" target="_blank">
							<?php 
    \esc_html_e( 'here', 'events-optimizer' );
    ?></a>).
							</td>
							</tr>						
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Add FAQ', 'events-optimizer' );
    ?></th>
							<td>
							  <input type="checkbox" name="openai_prompt_faq" id="openai_prompt_faq" value="true" <?php 
    if ( get_option( 'teo_openai_prompt_faq' ) === 'true' ) {
        echo  "checked" ;
    }
    ?> <?php 
    echo  "disabled" ;
    ?> />
								<?php 
    \esc_html_e( '(improve your website user experience and increase your search engine rankings)', 'events-optimizer' );
    ?>
							</td>
							</tr>
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Add headings', 'events-optimizer' );
    ?></th>
							<td>
							  <input type="checkbox" name="openai_prompt_headings" id="openai_prompt_headings" value="true" <?php 
    if ( get_option( 'teo_openai_prompt_headings' ) === 'true' ) {
        echo  "checked" ;
    }
    ?> <?php 
    echo  "disabled" ;
    ?> />
								<?php 
    \esc_html_e( '(works only when "Output as HTML" is activated!)', 'events-optimizer' );
    ?>
							</td>
							</tr>
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Output as HTML', 'events-optimizer' );
    ?></th>
							<td>
							  <input type="checkbox" name="openai_prompt_html" id="openai_prompt_html" value="true" <?php 
    if ( get_option( 'teo_openai_prompt_html' ) === 'true' ) {
        echo  "checked" ;
    }
    ?> <?php 
    echo  "disabled" ;
    ?> />
								<?php 
    \esc_html_e( '(required if FAQ or headings added)', 'events-optimizer' );
    ?>
							</td>
							</tr>													
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Replace existing content', 'events-optimizer' );
    ?></th>
							<td>
							  <input type="checkbox" name="openai_replace_content" id="openai_replace_content" value="true" <?php 
    if ( get_option( 'teo_openai_replace_content' ) === 'true' ) {
        echo  "checked" ;
    }
    ?> <?php 
    echo  "disabled" ;
    ?> />
								<?php 
    \esc_html_e( '(replace existing content or add new content to it)', 'events-optimizer' );
    ?>
							</td>
							</tr>
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Hide Generate description buttons', 'events-optimizer' );
    ?></th>
							<td>
							  <input type="checkbox" name="openai_hide_generate_description_buttons" id="openai_hide_generate_description_buttons" value="true" <?php 
    if ( get_option( 'teo_openai_hide_generate_description_buttons' ) === 'true' ) {
        echo  "checked" ;
    }
    ?> <?php 
    echo  "disabled" ;
    ?> />
							  	<?php 
    \esc_html_e( '(hide buttons in the single event, venue and organizer form)', 'events-optimizer' );
    ?>
							</td>
							</tr>							
						</table>									
						<br>

						<br><h3><?php 
    \esc_html_e( 'Security', 'events-optimizer' );
    ?></h3>				
						<div id="modern-tribe-info">												
							<li><?php 
    \esc_html_e( 'The Google Safe Browsing technology offers a way to vaildate if a website is not safe possibly', 'events-optimizer' );
    ?> 
							<?php 
    \esc_html_e( '(learn more about in the', 'events-optimizer' );
    ?> <a href="https://transparencyreport.google.com/safe-browsing/overview" target="_blank">Googles Transparency Report).</a>
							<?php 
    \esc_html_e( 'By enabling this option the URLs of venues, organizers and events will be verified.', 'events-optimizer' );
    ?>
							<?php 
    \esc_html_e( 'This can slightly slow down the analysis process.', 'events-optimizer' );
    ?><br></li>
							<li><?php 
    \esc_html_e( 'Check if URLs have missing https / http or are redirected / permanently moved.', 'events-optimizer' );
    ?></li>
						</div>
						<table class="form-table">	
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Verify URLs with Google Safe Browsing', 'events-optimizer' );
    ?></th>
							<td>
							  <input type="checkbox" name="google_safe_browsing" id="google_safe_browsing" value="true" <?php 
    if ( get_option( 'teo_google_safe_browsing' ) === 'true' ) {
        echo  "checked" ;
    }
    ?> <?php 
    echo  "disabled" ;
    ?> />
								<?php 
    \esc_html_e( '(recommended to turn on)', 'events-optimizer' );
    ?>
							</td>
							</tr>
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Verify URLs on missing https/ http', 'events-optimizer' );
    ?></th>
							<td>
							  <input type="checkbox" name="validate_url_https" id="validate_url_https" value="true" <?php 
    if ( get_option( 'teo_validate_url_https' ) === 'true' ) {
        echo  "checked" ;
    }
    ?> <?php 
    echo  "disabled" ;
    ?> />
								<?php 
    \esc_html_e( '(recommended to turn on)', 'events-optimizer' );
    ?>
							</td>
							</tr>	
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Verify if URLs are redirected or permanently moved', 'events-optimizer' );
    ?></th>
							<td>
							  <input type="checkbox" name="validate_url_redirected_moved" id="validate_url_redirected_moved" value="true" <?php 
    if ( get_option( 'teo_validate_url_redirected_moved' ) === 'true' ) {
        echo  "checked" ;
    }
    ?> <?php 
    echo  "disabled" ;
    ?> />
								<?php 
    \esc_html_e( '(recommended to turn on)', 'events-optimizer' );
    ?>
							</td>
							</tr>														
						</table>

						<br><h3><?php 
    \esc_html_e( 'Google Places', 'events-optimizer' );
    ?></h3>
						<div id="modern-tribe-info">
							<?php 
    \esc_html_e( 'To automatically optimize your data the Google Places API is used.', 'events-optimizer' );
    ?><br>
							<li><?php 
    \esc_html_e( 'To set up and configure your API key check out the ', 'events-optimizer' );
    ?><a href="https://developers.google.com/maps/documentation/places/web-service" target="_blank">
							<?php 
    \esc_html_e( 'Google Places API documentation.', 'events-optimizer' );
    ?></a></li>
							<li><?php 
    \esc_html_e( 'Activate the Google PLACES API for the relevant project in the ', 'events-optimizer' );
    ?><a href="https://console.cloud.google.com/" target="_blank">
							<?php 
    \esc_html_e( 'Google Cloud Console.', 'events-optimizer' );
    ?></a>
							<?php 
    \esc_html_e( 'The easiest way for this is to use your (possibly already existing) Google MAPS API key from the ', 'events-optimizer' );
    ?>
							<?php 
    // Path to Events Calendar settings changed from V5.9 to V6.0
    $plugin_arr = get_plugin_data( WP_PLUGIN_DIR . '/the-events-calendar/the-events-calendar.php', false, false );
    
    if ( $plugin_arr['Version'] < 6 ) {
        echo  '<a href="/wp-admin/edit.php?page=tribe-common&tab=addons&post_type=tribe_events" target="_blank">' ;
    } else {
        echo  '<a href="/wp-admin/edit.php?page=tec-events-settings&tab=addons&post_type=tribe_events" target="_blank">' ;
    }
    
    ?>
							<?php 
    \esc_html_e( 'Events Calendar settings.', 'events-optimizer' );
    ?></a></li>
							<li><?php 
    \esc_html_e( 'Before using it please review the', 'events-optimizer' );
    ?><a href="https://developers.google.com/maps/billing/gmp-billing" target="_blank">
							<?php 
    \esc_html_e( 'pricing of the Google Places API.', 'events-optimizer' );
    ?></a></li>					
						</div>	
															
						<table class="form-table">
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Google Places API key', 'events-optimizer' );
    ?></th>
							<td><input type="text" name="google_places_api_key" id="google_places_api_key" size="44" value="<?php 
    echo  esc_attr( get_option( 'teo_google_places_api_key' ) ) ;
    ?>"  <?php 
    echo  "disabled" ;
    ?> />
							</td>						
							</tr>
							<th scope="row"><?php 
    \esc_html_e( 'Language (ISO code)', 'events-optimizer' );
    ?></th>
							<td><input type="text" name="google_places_language" id="google_places_language" size="5" value="<?php 
    echo  esc_attr( get_option( 'teo_google_places_language' ) ) ;
    ?>"  <?php 
    echo  "disabled" ;
    ?> />
								<?php 
    \esc_html_e( '(according to', 'events-optimizer' );
    ?> 
								<a href="https://developers.google.com/maps/faq#languagesupport" target="_blank">
								<?php 
    \esc_html_e( 'Google Places API)', 'events-optimizer' );
    ?>
								</a>
							</td>

							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Phone number format', 'events-optimizer' );
    ?></th>
							<td>
							<select name="phone_number_format" id="phone_number_format"  <?php 
    echo  "disabled" ;
    ?> >
								<option value="international_phone_number" <?php 
    echo  ( $phone_number_format === "international_phone_number" ? 'selected="selected"' : '' ) ;
    ?>><?php 
    \esc_html_e( 'International', 'events-optimizer' );
    ?></option>
								<option value="formatted_phone_number" <?php 
    echo  ( $phone_number_format === "formatted_phone_number" ? 'selected="selected"' : '' ) ;
    ?>><?php 
    \esc_html_e( 'Local', 'events-optimizer' );
    ?></option>
							</select>
							<?php 
    \esc_html_e( '(int. = +49..., local = (089) 1234...)', 'events-optimizer' );
    ?>
							</td>
							</tr>

							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Address format', 'events-optimizer' );
    ?></th>
							<td>
							<select name="address_format" id="address_format"  <?php 
    echo  "disabled" ;
    ?> >
								<option value="street_number" <?php 
    echo  ( $address_format === "street_number" ? 'selected="selected"' : '' ) ;
    ?>><?php 
    \esc_html_e( 'Street name + number', 'events-optimizer' );
    ?></option>
								<option value="number_street" <?php 
    echo  ( $address_format === "number_street" ? 'selected="selected"' : '' ) ;
    ?>><?php 
    \esc_html_e( 'Street number + name', 'events-optimizer' );
    ?></option>
							</select>
							<?php 
    \esc_html_e( '(Sesame street 42 vs. 42th Sesame street)', 'events-optimizer' );
    ?>
							</td>
							</tr>

							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Preselection', 'events-optimizer' );
    ?></th>
							<td>
							<select name="preselection" id="preselection"  <?php 
    echo  "disabled" ;
    ?> >
								<option value="auto_completion" <?php 
    echo  ( $preselection === "auto_completion" ? 'selected="selected"' : '' ) ;
    ?>><?php 
    \esc_html_e( 'Auto completion of missing data', 'events-optimizer' );
    ?></option>
								<option value="full_replacement" <?php 
    echo  ( $preselection === "full_replacement" ? 'selected="selected"' : '' ) ;
    ?>><?php 
    \esc_html_e( 'Full replacement via Google Places', 'events-optimizer' );
    ?></option>
							</select>
							</td>
							</tr>
						</table>

						<br><h3><?php 
    \esc_html_e( 'Fetch data from Google Places within single organizer form', 'events-optimizer' );
    ?></h3>				
						<div id="modern-tribe-info">
							<strong><?php 
    \esc_html_e( 'Only for experienced users who are familiar with Custom Fields (no support provided for this feature!)', 'events-optimizer' );
    ?></strong><br><br>
							<?php 
    \esc_html_e( 'In the single organizer form data from', 'events-optimizer' );
    ?> 
							<a href="https://developers.google.com/maps/documentation/places/web-service/details?hl=de#Place" target="_blank">
							<?php 
    \esc_html_e( 'Google Places', 'events-optimizer' );
    ?></a> 
							<?php 
    \esc_html_e( 'can be fetched.', 'events-optimizer' );
    ?><br>
							<li><?php 
    \esc_html_e( 'If you add custom fields with the following ids to the single organizer form they will be populated with the corresponding data from Google Places:', 'events-optimizer' );
    ?><br>		
							<?php 
    \esc_html_e( 'OrganizerAddress, OrganizerCity, OrganizerZip, OrganizerProvince, OrganizerState, OrganizerCountry, OrganizerCurrentOpeningHours, OrganizerRating', 'events-optimizer' );
    ?></li>												
							<li><?php 
    \esc_html_e( 'The full address can be populated in any input or textarea field. Enter the field id below.', 'events-optimizer' );
    ?></li>				
						</div>

						<table class="form-table">	
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Field id for full address', 'events-optimizer' );
    ?></th>
							<td><input type="text" name="element_id" id="element_id" size="30" value="<?php 
    echo  esc_attr( get_option( 'teo_element_id' ) ) ;
    ?>"  <?php 
    echo  "disabled" ;
    ?> />
								<?php 
    \esc_html_e( '(e.g. "content" or "excerpt")', 'events-optimizer' );
    ?>
							</td>
							</tr>
						</table>

						<br><h3><?php 
    \esc_html_e( 'Google Images', 'events-optimizer' );
    ?></h3>
						<div id="modern-tribe-info">
							<?php 
    \esc_html_e( 'To automatically add images with Google Images data the DataForSEO API is used which you can configure below.', 'events-optimizer' );
    ?><br>
							<?php 
    \esc_html_e( 'You can search for any images, but please use them only in accordance with the copyright law as you can be sued!', 'events-optimizer' );
    ?>
						</div>
															
						<table class="form-table">
							<tr valign="top">					
							<th scope="row"><?php 
    \esc_html_e( 'Maximum images to search', 'events-optimizer' );
    ?></th>
							<td><input type="text" name="max_images" id="max_images" size="1"  value="<?php 
    echo  esc_attr( get_option( 'teo_max_images' ) ) ;
    ?>"  <?php 
    echo  "disabled" ;
    ?> />
							</td>
							</tr>	
						</table>									
						<br>

						<br><h3><?php 
    \esc_html_e( 'Google Events', 'events-optimizer' );
    ?><span class='beta_version'>BETA</span></h3>
						<div id="modern-tribe-info">
							<?php 
    \esc_html_e( 'To automatically add events with Google Events data the DataForSEO API is used.', 'events-optimizer' );
    ?>
							<li><?php 
    \esc_html_e( 'To configure and set it up check out', 'events-optimizer' );
    ?> <a href="https://developers.google.com/maps/documentation/places/web-service" target="_blank">
							<a href="https://dataforseo.com/?aff=142846" target="_blank"><?php 
    \esc_html_e( 'dataforseo.com', 'events-optimizer' );
    ?></a> 
							<?php 
    \esc_html_e( '(free trial with affiliate link).', 'events-optimizer' );
    ?></li>
							<li><?php 
    \esc_html_e( 'You will find your API login and password in your', 'events-optimizer' );
    ?> <a href="https://app.dataforseo.com/api-dashboard/?aff=142846" target="_blank">
							<?php 
    \esc_html_e( 'DataForSEO Dashboard.', 'events-optimizer' );
    ?></a>
							<?php 
    \esc_html_e( 'Before using it please review the', 'events-optimizer' );
    ?> <a href="https://dataforseo.com/pricing/serp/serp-api/?aff=142846" target="_blank">
							<?php 
    \esc_html_e( 'pricing of the DataForSEO.', 'events-optimizer' );
    ?></a></li>
							<li><?php 
    \esc_html_e( 'Try out the search at', 'events-optimizer' );
    ?> 
							<a href="https://www.google.com/search?q=event&ibp=htl;events&rciv=evn#fpstate=tldetail" target="_blank">
							<?php 
    \esc_html_e( 'Google Events.', 'events-optimizer' );
    ?></a></li>
							<li><?php 
    \esc_html_e( 'It is recommended to add new events in a regular rhythm, e.g. every month. This can avoid duplicates.', 'events-optimizer' );
    ?></li>
						</div>
															
						<table class="form-table">
							<tr valign="top">							
							<th scope="row"><?php 
    \esc_html_e( 'DataForSEO API login', 'events-optimizer' );
    ?></th>
							<td><input type="text" name="dataforseo_api_login" id="dataforseo_api_login" size="30"  value="<?php 
    echo  esc_attr( get_option( 'teo_dataforseo_api_login' ) ) ;
    ?>"  <?php 
    echo  "disabled" ;
    ?> />
							</td>
							</tr>	
							<tr valign="top">							
							<th scope="row"><?php 
    \esc_html_e( 'DataForSEO API password', 'events-optimizer' );
    ?></th>
							<td><input type="text" name="dataforseo_api_password" id="dataforseo_api_password" size="30"  value="<?php 
    echo  esc_attr( get_option( 'teo_dataforseo_api_password' ) ) ;
    ?>"  <?php 
    echo  "disabled" ;
    ?> />
							</td>
							</tr>							
							<tr valign="top">							
							<th scope="row"><?php 
    \esc_html_e( 'Date range', 'events-optimizer' );
    ?></th>
							<td>
							<select name="dataforseo_date_range" id="dataforseo_date_range" <?php 
    echo  "disabled" ;
    ?> >  
							<?php 
    $date_range_options = array(
        'all'        => __( 'All upcoming events', 'events-optimizer' ),
        'today'      => __( 'Today', 'events-optimizer' ),
        'tomorrow'   => __( 'Tomorrow', 'events-optimizer' ),
        'weekend'    => __( 'This weekend', 'events-optimizer' ),
        'week'       => __( 'This week', 'events-optimizer' ),
        'next_week'  => __( 'Next week', 'events-optimizer' ),
        'month'      => __( 'This month', 'events-optimizer' ),
        'next_month' => __( 'Next month', 'events-optimizer' ),
    );
    foreach ( $date_range_options as $dr_value => $dr_decription ) {
        
        if ( $dr_value === get_option( 'teo_dataforseo_date_range' ) ) {
            echo  '<option value="' . $dr_value . '" selected>' . $dr_decription . '</option>' ;
        } else {
            echo  '<option value="' . $dr_value . '">' . $dr_decription . '</option>' ;
        }
    
    }
    ?>
							</select>  
							</td>
							</tr>
							<tr valign="top">							
							<th scope="row"><?php 
    \esc_html_e( 'Language (ISO code)', 'events-optimizer' );
    ?></th>
							<td><input type="text" name="dataforseo_language_code" id="dataforseo_language_code" size="5"  value="<?php 
    echo  esc_attr( get_option( 'teo_dataforseo_language_code' ) ) ;
    ?>"  <?php 
    echo  "disabled" ;
    ?> />
							<?php 
    \esc_html_e( '(according to', 'events-optimizer' );
    ?> 
							<a href="https://cdn.dataforseo.com/v3/languages/languages_serp_google_2022_01_17.csv">
							<?php 
    \esc_html_e( 'this list from DataForSEO)', 'events-optimizer' );
    ?>
							</a>
							</td>
							</tr>							
							<tr valign="top">							
							<th scope="row"><?php 
    \esc_html_e( 'Location code', 'events-optimizer' );
    ?></th>
							<td><input type="text" name="dataforseo_location_code" id="dataforseo_location_code" size="10"  value="<?php 
    echo  esc_attr( get_option( 'teo_dataforseo_location_code' ) ) ;
    ?>"  <?php 
    echo  "disabled" ;
    ?> />
							<?php 
    \esc_html_e( '(according to', 'events-optimizer' );
    ?>
							<a href="https://www.heinrich.biz/wp-content/uploads/optimizer/locations_codes.csv">
							<?php 
    \esc_html_e( 'this list from DataForSEO)', 'events-optimizer' );
    ?>
							</a>
							</td>
							</tr>
						</table>							
						<table class="form-table">
							<div id="modern-tribe-info">
								<?php 
    \esc_html_e( 'The following events are already added from Google Events. They will be ignored when searching for additional events.', 'events-optimizer' );
    ?><br>
							</div>			
							<tr valign="top">
							<th scope="row">
								<input type="button" name="action" class="button" id="reset_objects_added_button" <?php 
    echo  'disabled' ;
    ?> value="<?php 
    esc_html_e( 'Reset list', 'events-optimizer' );
    ?>" />
							</th>
							<td>
								<?php 
    $ignored_and_added_objects = unserialize( get_option( 'teo_ignored_and_added_objects' ) );
    
    if ( empty($ignored_and_added_objects) === false ) {
        echo  '<p><strong>' . __( 'Google Events', 'events-optimizer' ) . '</strong></p>' ;
        foreach ( $ignored_and_added_objects as $object ) {
            if ( $object['status'] === 'added' ) {
                echo  '<div id="object_' . $object['id'] . '">
												  		<a href="/wp-admin/post.php?post=' . $object['post_id'] . '&action=edit" target="_blank">' . get_the_title( $object['post_id'] ) . ' (' . date_format( date_create( $object['start_datetime_gp'] ), 'm.d.Y, H:i' ) . ')</a>
												  		<a href="/wp-admin/post.php?post=' . $object['post_id'] . '&action=edit" target="_blank"><img src="' . plugin_dir_url( __FILE__ ) . 'assets/images/external-link.png" class="external-link-icon" alt="External Link"></a>&nbsp;
														<img src="' . plugin_dir_url( __FILE__ ) . 'assets/images/delete-x-red.svg" class="delete-icon" alt="remove object from list" onclick="remove_object(' . $object['id'] . ');" />&nbsp;&nbsp;&nbsp;
													</div>' ;
            }
        }
    } else {
        echo  '--' ;
    }
    
    ?>
							</td>							
							</tr>						
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Show added events', 'events-optimizer' );
    ?></th>
							<td>
							  <input type="checkbox" name="show_events_added" id="show_events_added" value="true" <?php 
    if ( get_option( 'teo_show_events_added' ) === 'true' ) {
        echo  "checked" ;
    }
    ?> <?php 
    echo  "disabled" ;
    ?> />
							</td>
							</tr>																																																						
						</table>
					

						<br><h3><?php 
    \esc_html_e( 'Ignored objects', 'events-optimizer' );
    ?></h3>				
						<div id="modern-tribe-info">
							<?php 
    \esc_html_e( 'The following objects are ignored when searching for issues.', 'events-optimizer' );
    ?><br>
						</div>
						<table class="form-table">	
							<tr valign="top">
							<th scope="row">
								<input type="button" name="action" class="button" id="reset_objects_ignored_button" <?php 
    echo  'disabled' ;
    ?> value="<?php 
    esc_html_e( 'Reset list', 'events-optimizer' );
    ?>" />
							</th>
							<td>
			
							  <div class="row">					
								<?php 
    $ignored_and_added_objects = unserialize( get_option( 'teo_ignored_and_added_objects' ) );
    
    if ( empty($ignored_and_added_objects) === false ) {
        echo  '<p><strong>' . __( 'Venues', 'events-optimizer' ) . '</strong></p>' ;
        foreach ( $ignored_and_added_objects as $object ) {
            if ( $object['status'] === 'ignored' and $object['type'] === 'tribe_venue' ) {
                echo  '<div id="object_' . $object['id'] . '">
												  		<a href="/wp-admin/post.php?post=' . $object['post_id'] . '&action=edit" target="_blank">' . get_the_title( $object['post_id'] ) . '</a>
														<a href="/wp-admin/post.php?post=' . $object['post_id'] . '&action=edit" target="_blank"><img src="' . plugin_dir_url( __FILE__ ) . 'assets/images/external-link.png" class="external-link-icon" alt="External Link"></a>&nbsp;
														<img src="' . plugin_dir_url( __FILE__ ) . 'assets/images/delete-x-red.svg" class="delete-icon" alt="remove object from list" onclick="remove_object(' . $object['id'] . ');" />&nbsp;&nbsp;&nbsp;
													</div>' ;
            }
        }
        echo  '</div>' ;
        echo  '<br><p><strong>' . __( 'Organizers', 'events-optimizer' ) . '</strong></p>' ;
        foreach ( $ignored_and_added_objects as $object ) {
            if ( $object['status'] === 'ignored' and $object['type'] === 'tribe_organizer' ) {
                echo  '<div id="object_' . $object['id'] . '">
												  		<a href="/wp-admin/post.php?post=' . $object['post_id'] . '&action=edit" target="_blank">' . get_the_title( $object['post_id'] ) . '</a>
														<a href="/wp-admin/post.php?post=' . $object['post_id'] . '&action=edit" target="_blank"><img src="' . plugin_dir_url( __FILE__ ) . 'assets/images/external-link.png" class="external-link-icon" alt="External Link"></a>&nbsp;
														<img src="' . plugin_dir_url( __FILE__ ) . 'assets/images/delete-x-red.svg" class="delete-icon" alt="remove object from list" onclick="remove_object(' . $object['id'] . ');" />&nbsp;&nbsp;&nbsp;
													</div>' ;
            }
        }
        echo  '<br><p><strong>' . __( 'Events', 'events-optimizer' ) . '</strong></p>' ;
        foreach ( $ignored_and_added_objects as $object ) {
            if ( $object['status'] === 'ignored' and $object['type'] === 'tribe_events' ) {
                
                if ( $object['post_id'] != '' ) {
                    echo  '<div id="object_' . $object['id'] . '">
															<a href="/wp-admin/post.php?post=' . $object['post_id'] . '&action=edit" target="_blank">' . get_the_title( $object['post_id'] ) . '</a>
															<a href="/wp-admin/post.php?post=' . $object['post_id'] . '&action=edit" target="_blank"><img src="' . plugin_dir_url( __FILE__ ) . 'assets/images/external-link.png" class="external-link-icon" alt="External Link"></a>&nbsp;
															<img src="' . plugin_dir_url( __FILE__ ) . 'assets/images/delete-x-red.svg" class="delete-icon" alt="remove object from list" onclick="remove_object(' . $object['id'] . ');" />&nbsp;&nbsp;&nbsp;
														  </div>' ;
                } else {
                    echo  '<div id="object_' . $object['id'] . '">' . $object['title_gp'] . ' (' . date_format( date_create( $object['start_datetime_gp'] ), get_option( 'date_format' ) . ", H:i" ) . ')' . ' <img src="' . plugin_dir_url( __FILE__ ) . 'assets/images/delete-x-red.svg" class="delete-icon" alt="remove object from list" onclick="remove_object(' . $object['id'] . ');" />&nbsp;&nbsp;&nbsp;
													</div>' ;
                }
            
            }
        }
    } else {
        echo  '--' ;
    }
    
    ?>
	
								</div>						
							</td>
							</tr>							
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Show ignored objects', 'events-optimizer' );
    ?></th>
							<td>
							  <input type="checkbox" name="show_ignored_objects" id="show_ignored_objects" value="true" <?php 
    if ( get_option( 'teo_show_ignored_objects' ) === 'true' ) {
        echo  "checked" ;
    }
    ?> <?php 
    echo  "disabled" ;
    ?> />
							</td>
							</tr>
						</table>

						<br><h3><?php 
    \esc_html_e( 'Ignored fields', 'events-optimizer' );
    ?></h3>				
						<div id="modern-tribe-info">
							<?php 
    \esc_html_e( 'All fields checked will be ignored when searching for invalid data.', 'events-optimizer' );
    ?><br>				
						</div>

						<br><h3><?php 
    \esc_html_e( 'Venues', 'events-optimizer' );
    ?></h3>
						<table class="form-table">	
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Description', 'events-optimizer' );
    ?></th>
							<td><input type="checkbox" name="ignored_field_venue_description" id="ignored_field_venue_description" value="true" <?php 
    if ( get_option( 'teo_ignored_field_venue_description' ) === 'true' ) {
        echo  "checked" ;
    }
    ?> <?php 
    echo  "disabled" ;
    ?> />
							</td>
							</tr>							
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Address', 'events-optimizer' );
    ?></th>
							<td><input type="checkbox" name="ignored_field_venue_address" id="ignored_field_venue_address" value="true" <?php 
    if ( get_option( 'teo_ignored_field_venue_address' ) === 'true' ) {
        echo  "checked" ;
    }
    ?> <?php 
    echo  "disabled" ;
    ?> />
							</td>
							</tr>
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'ZIP', 'events-optimizer' );
    ?></th>
							<td><input type="checkbox" name="ignored_field_venue_zip" id="ignored_field_venue_zip" value="true" <?php 
    if ( get_option( 'teo_ignored_field_venue_zip' ) === 'true' ) {
        echo  "checked" ;
    }
    ?> <?php 
    echo  "disabled" ;
    ?> />
							</td>
							</tr>	
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'City', 'events-optimizer' );
    ?></th>
							<td><input type="checkbox" name="ignored_field_venue_city" id="ignored_field_venue_city" value="true" <?php 
    if ( get_option( 'teo_ignored_field_venue_city' ) === 'true' ) {
        echo  "checked" ;
    }
    ?> <?php 
    echo  "disabled" ;
    ?> />
							</td>
							</tr>
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Province', 'events-optimizer' );
    ?></th>
							<td><input type="checkbox" name="ignored_field_venue_province" id="ignored_field_venue_province" value="true" <?php 
    if ( get_option( 'teo_ignored_field_venue_province' ) === 'true' ) {
        echo  "checked" ;
    }
    ?> <?php 
    echo  "disabled" ;
    ?> />
							</td>
							</tr>
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'State', 'events-optimizer' );
    ?></th>
							<td><input type="checkbox" name="ignored_field_venue_state" id="ignored_field_venue_state" value="true" <?php 
    if ( get_option( 'teo_ignored_field_venue_state' ) === 'true' ) {
        echo  "checked" ;
    }
    ?> <?php 
    echo  "disabled" ;
    ?> />
							</td>
							</tr>
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Country', 'events-optimizer' );
    ?></th>
							<td><input type="checkbox" name="ignored_field_venue_country" id="ignored_field_venue_country" value="true" <?php 
    if ( get_option( 'teo_ignored_field_venue_country' ) === 'true' ) {
        echo  "checked" ;
    }
    ?> <?php 
    echo  "disabled" ;
    ?> />
							</td>
							</tr>	
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Phone', 'events-optimizer' );
    ?></th>
							<td><input type="checkbox" name="ignored_field_venue_phone" id="ignored_field_venue_phone" value="true" <?php 
    if ( get_option( 'teo_ignored_field_venue_phone' ) === 'true' ) {
        echo  "checked" ;
    }
    ?> <?php 
    echo  "disabled" ;
    ?> />
							</td>
							</tr>	
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Website', 'events-optimizer' );
    ?></th>
							<td><input type="checkbox" name="ignored_field_venue_website" id="ignored_field_venue_website" value="true" <?php 
    if ( get_option( 'teo_ignored_field_venue_website' ) === 'true' ) {
        echo  "checked" ;
    }
    ?> <?php 
    echo  "disabled" ;
    ?> />
							</td>
							</tr>
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Business Status', 'events-optimizer' );
    ?></th>
							<td><input type="checkbox" name="ignored_field_venue_business_status" id="ignored_field_venue_business_status" value="true" <?php 
    if ( get_option( 'teo_ignored_field_venue_business_status' ) === 'true' ) {
        echo  "checked" ;
    }
    ?> <?php 
    echo  "disabled" ;
    ?> />
							</td>
							</tr>																											
						</table>

						<br><h3><?php 
    \esc_html_e( 'Organizers', 'events-optimizer' );
    ?></h3>
						<table class="form-table">	
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Description', 'events-optimizer' );
    ?></th>
							<td><input type="checkbox" name="ignored_field_organizer_description" id="ignored_field_organizer_description" value="true" <?php 
    if ( get_option( 'teo_ignored_field_organizer_description' ) === 'true' ) {
        echo  "checked" ;
    }
    ?> <?php 
    echo  "disabled" ;
    ?> />
							</td>
							</tr>								
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Phone', 'events-optimizer' );
    ?></th>
							<td><input type="checkbox" name="ignored_field_organizer_phone" id="ignored_field_organizer_phone" value="true" <?php 
    if ( get_option( 'teo_ignored_field_organizer_phone' ) === 'true' ) {
        echo  "checked" ;
    }
    ?> <?php 
    echo  "disabled" ;
    ?> />
							</td>
							</tr>	
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Website', 'events-optimizer' );
    ?></th>
							<td><input type="checkbox" name="ignored_field_organizer_website" id="ignored_field_organizer_website" value="true" <?php 
    if ( get_option( 'teo_ignored_field_organizer_website' ) === 'true' ) {
        echo  "checked" ;
    }
    ?> <?php 
    echo  "disabled" ;
    ?> />
							</td>
							</tr>																				
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Mail', 'events-optimizer' );
    ?></th>
							<td><input type="checkbox" name="ignored_field_organizer_mail" id="ignored_field_organizer_mail" value="true" <?php 
    if ( get_option( 'teo_ignored_field_organizer_mail' ) === 'true' ) {
        echo  "checked" ;
    }
    ?> <?php 
    echo  "disabled" ;
    ?> />
							</td>
							</tr>
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Business Status', 'events-optimizer' );
    ?></th>
							<td><input type="checkbox" name="ignored_field_organizer_business_status" id="ignored_field_organizer_business_status" value="true" <?php 
    if ( get_option( 'teo_ignored_field_organizer_business_status' ) === 'true' ) {
        echo  "checked" ;
    }
    ?> <?php 
    echo  "disabled" ;
    ?> />
							</td>
							</tr>	
						</table>

						<br><h3><?php 
    \esc_html_e( 'Events', 'events-optimizer' );
    ?></h3>
						<table class="form-table">	
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Description', 'events-optimizer' );
    ?></th>
							<td><input type="checkbox" name="ignored_field_event_description" id="ignored_field_event_description" value="true" <?php 
    if ( get_option( 'teo_ignored_field_event_description' ) === 'true' ) {
        echo  "checked" ;
    }
    ?> <?php 
    echo  "disabled" ;
    ?> />
							</td>
							</tr>		
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Start', 'events-optimizer' );
    ?></th>
							<td><input type="checkbox" name="ignored_field_event_start" id="ignored_field_event_start" value="true" <?php 
    if ( get_option( 'teo_ignored_field_event_start' ) === 'true' ) {
        echo  "checked" ;
    }
    ?> <?php 
    echo  "disabled" ;
    ?> />
							</td>
							</tr>	
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'End', 'events-optimizer' );
    ?></th>
							<td><input type="checkbox" name="ignored_field_event_end" id="ignored_field_event_end" value="true" <?php 
    if ( get_option( 'teo_ignored_field_event_end' ) === 'true' ) {
        echo  "checked" ;
    }
    ?> <?php 
    echo  "disabled" ;
    ?> />
							</td>
							</tr>								
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Venue', 'events-optimizer' );
    ?></th>
							<td><input type="checkbox" name="ignored_field_event_venue" id="ignored_field_event_venue" value="true" <?php 
    if ( get_option( 'teo_ignored_field_event_venue' ) === 'true' ) {
        echo  "checked" ;
    }
    ?> <?php 
    echo  "disabled" ;
    ?> />
							</td>
							</tr>																	
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Organizer', 'events-optimizer' );
    ?></th>
							<td><input type="checkbox" name="ignored_field_event_organizer" id="ignored_field_event_organizer" value="true" <?php 
    if ( get_option( 'teo_ignored_field_event_organizer' ) === 'true' ) {
        echo  "checked" ;
    }
    ?> <?php 
    echo  "disabled" ;
    ?> />
							</td>
							</tr>
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Website', 'events-optimizer' );
    ?></th>
							<td><input type="checkbox" name="ignored_field_event_website" id="ignored_field_event_website" value="true" <?php 
    if ( get_option( 'teo_ignored_field_event_website' ) === 'true' ) {
        echo  "checked" ;
    }
    ?> <?php 
    echo  "disabled" ;
    ?> />
							</td>
							</tr>	
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Currency symbol', 'events-optimizer' );
    ?></th>
							<td><input type="checkbox" name="ignored_field_event_currency_symbol" id="ignored_field_event_currency_symbol" value="true" <?php 
    if ( get_option( 'teo_ignored_field_event_currency_symbol' ) === 'true' ) {
        echo  "checked" ;
    }
    ?> <?php 
    echo  "disabled" ;
    ?> />
							</td>
							</tr>
							<tr valign="top">
							<th scope="row"><?php 
    \esc_html_e( 'Cost', 'events-optimizer' );
    ?></th>
							<td><input type="checkbox" name="ignored_field_event_cost" id="ignored_field_event_cost" value="true" <?php 
    if ( get_option( 'teo_ignored_field_event_cost' ) === 'true' ) {
        echo  "checked" ;
    }
    ?> <?php 
    echo  "disabled" ;
    ?> />
							</td>
							</tr>
						</table>

						<input type='submit' name='action' class='button-primary' id='save_settings' value='<?php 
    esc_html_e( 'Save changes', 'events-optimizer' );
    ?>' />
					</div>
				</form>
				<p>
					<div id='show_tab_settings'></div>
					<div id='loading_tab_settings'><?php 
    \esc_attr_e( 'SAVING', 'events-optimizer' );
    ?></div>
				</p>
			</div>
		</div>

		<!-- jQuery: Fade in/out buttons and call php functions to analyse data -->	
		<script type='text/javascript'>
		
			// Wait until the whole document has fully loaded and then do everything inside here ...
			jQuery(document).ready(function(){		
				jQuery('#venues_with_invalid_data').on('click', function()  { // Execute function when button is clicked
					jQuery("#venues_with_invalid_data").attr("disabled", true); // Disable button while analyzing
					jQuery.ajax({
						type: "post",
						url: "admin-ajax.php",
						data: { 
							action: 'venues_wid', // call function "Venues with invalid data"
							_ajax_nonce: '<?php 
    echo  $nonce ;
    ?>' 
						},
						beforeSend: function() {
							jQuery("#loading_tab_venues").fadeIn('fast'); // Fade in loading icon and "analyzing" text
							jQuery("#show_results_venues").fadeOut("fast"); // Hide results otherwise they are show on the other tabs!
						},
						success: function(html){ // Function is called and if data is retrieved, store it in variable "html"
							jQuery("#loading_tab_venues").fadeOut('slow'); // Fade out loading icon and "analyzing" text
							jQuery("#show_results_venues").html( html ); // Show the html inside "show_results_venues" div
							jQuery("#show_results_venues").fadeIn("fast"); // Fade in the results
							jQuery("#venues_with_invalid_data").attr("disabled", false); // Enable button after analyzing
						}
					}); //close jQuery.ajax
					return false;
				});	

				jQuery('#duplicate_venues').on('click', function()  {
					jQuery("#duplicate_venues").attr("disabled", true);		
					jQuery.ajax({
						type: "post",
						url: "admin-ajax.php",
						data: { 
							action: 'duplicate_ven', // call function "Duplicate Venues"
							_ajax_nonce: '<?php 
    echo  $nonce ;
    ?>' 
						},
						beforeSend: function() {
							jQuery("#loading_tab_venues").fadeIn('fast');
							jQuery("#show_results_venues").fadeOut("fast");
						},
						success: function(html){
							jQuery("#loading_tab_venues").fadeOut('slow');
							jQuery("#show_results_venues").html( html );
							jQuery("#show_results_venues").fadeIn("fast");
							jQuery("#duplicate_venues").attr("disabled", false);
						}
					});
					return false;					
				});
				
				jQuery('#venues_google_places_optimization').on('click', function()  {
					jQuery("#venues_google_places_optimization").attr("disabled", true);			
					jQuery.ajax({
						type: "post",
						url: "admin-ajax.php",
						data: { 
							action: 'venues_optimization_wgp', // call function "Venues optimization with Google Places"
							_ajax_nonce: '<?php 
    echo  $nonce ;
    ?>' 
						},
						beforeSend: function() {
							jQuery("#loading_tab_venues").fadeIn('fast');
							jQuery("#show_results_venues").fadeOut("fast");
						},
						success: function(html){
							jQuery("#loading_tab_venues").fadeOut('slow');
							jQuery("#show_results_venues").html( html );
							jQuery("#show_results_venues").fadeIn("fast");
							jQuery("#venues_google_places_optimization").attr("disabled", false);						
						}
					});
					return false;					
				});

				jQuery('#organizers_with_invalid_data').on('click', function()  {
					jQuery("#organizers_with_invalid_data").attr("disabled", true);
					jQuery.ajax({
						type: "post",
						url: "admin-ajax.php",
						data: { 
							action: 'organizers_wid', // call function "Organizers with invalid data"
							_ajax_nonce: '<?php 
    echo  $nonce ;
    ?>' 
						},
						beforeSend: function() {
							jQuery("#loading_tab_organizers").fadeIn('fast');
							jQuery("#show_results_organizers").fadeOut("fast");
						},
						success: function(html){
							jQuery("#loading_tab_organizers").fadeOut('slow');
							jQuery("#show_results_organizers").html( html );
							jQuery("#show_results_organizers").fadeIn("fast");
							jQuery("#organizers_with_invalid_data").attr("disabled", false);
						}
					});
					return false;
				});	
	
				jQuery('#duplicate_organizers').on('click', function()  {
					jQuery("#duplicate_organizers").attr("disabled", true);		
					jQuery.ajax({
						type: "post",
						url: "admin-ajax.php",
						data: { 
							action: 'duplicate_org',  // call function "Duplicate Organizers"
							_ajax_nonce: '<?php 
    echo  $nonce ;
    ?>' 
						},
						beforeSend: function() {
							jQuery("#loading_tab_organizers").fadeIn('fast');
							jQuery("#show_results_organizers").fadeOut("fast");
						},
						success: function(html){
							jQuery("#loading_tab_organizers").fadeOut('slow');
							jQuery("#show_results_organizers").html( html );
							jQuery("#show_results_organizers").fadeIn("fast");
							jQuery("#duplicate_organizers").attr("disabled", false);
						}
					});
					return false;					
				});	

				
				jQuery('#organizers_google_places_optimization').on('click', function()  {
					jQuery("#organizers_google_places_optimization").attr("disabled", true);
					jQuery.ajax({
						type: "post",
						url: "admin-ajax.php",
						data: { 
							action: 'organizers_optimization_wgp', // call function "Organizers optimization with Google Places"
							_ajax_nonce: '<?php 
    echo  $nonce ;
    ?>' 
						},
						beforeSend: function() {
							jQuery("#loading_tab_organizers").fadeIn('fast');
							jQuery("#show_results_organizers").fadeOut("fast");
						},
						success: function(html){
							jQuery("#loading_tab_organizers").fadeOut('slow');
							jQuery("#show_results_organizers").html( html );
							jQuery("#show_results_organizers").fadeIn("fast");
							jQuery("#organizers_google_places_optimization").attr("disabled", false);
						}
					});
					return false;					
				})		

				jQuery('#events_with_invalid_data').on('click', function()  {
					jQuery("#events_with_invalid_data").attr("disabled", true);		
					jQuery.ajax({
						type: "post",
						url: "admin-ajax.php",
						data: { 
							action: 'events_wid', // call function "Events with invalid data"
							_ajax_nonce: '<?php 
    echo  $nonce ;
    ?>' 
						},
						beforeSend: function() {
							jQuery("#loading_tab_events").fadeIn('fast');
							jQuery("#show_results_events").fadeOut("fast");
						},
						success: function(html){
							jQuery("#loading_tab_events").fadeOut('slow');
							jQuery("#show_results_events").html( html );
							jQuery("#show_results_events").fadeIn("fast");
							jQuery("#events_with_invalid_data").attr("disabled", false);
						}
					});
					return false;					
				});	

				jQuery('#search_events_with_google_data').on('click', function()  {
					jQuery("#search_events_with_google_data").attr("disabled", true);		
					jQuery.ajax({
						type: "post",
						url: "admin-ajax.php",
						data: { 
							action: 'search_events_wgd', // call function "Search events with Google data"
							dataforseo_keyword:	escape( jQuery( '#dataforseo_keyword' ).val() ),
							dataforseo_depth:	escape( jQuery( '#dataforseo_depth' ).val() ),							
							_ajax_nonce: '<?php 
    echo  $nonce ;
    ?>' 
						},
						beforeSend: function() {
							//jQuery("#loading_tab_events").text('ANALYZING');							
							jQuery("#loading_tab_events").fadeIn('fast');
							jQuery("#show_results_events").fadeOut("fast");
						},
						success: function(html){
							jQuery("#loading_tab_events").fadeOut('slow');
							jQuery("#show_results_events").html( html );
							jQuery("#show_results_events").fadeIn("fast");
							jQuery("#search_events_with_google_data").attr("disabled", false);
						}
					});
					return false;					
				});	
								
				jQuery('#add_events_with_google_data').on('click', function()  {
					jQuery("#add_events_with_google_data").attr("disabled", true);		
					jQuery.ajax({
						type: "post",
						url: "admin-ajax.php",
						data: { 
							action: 'add_events_wgd', // call function "Add events from Google data"
							_ajax_nonce: '<?php 
    echo  $nonce ;
    ?>' 
						},
						beforeSend: function() {
							//jQuery("#loading_tab_events").text('DOWNLOAD SEARCH RESULTS');						
							jQuery("#loading_tab_events").fadeIn('fast');
							jQuery("#show_results_events").fadeOut("fast");
						},
						success: function(html){
							jQuery("#loading_tab_events").fadeOut('slow');
							jQuery("#show_results_events").html( html );
							jQuery("#show_results_events").fadeIn("fast");
							jQuery("#add_events_with_google_data").attr("disabled", false);
						}
					});
					return false;					
				});	

				jQuery('#save_settings').on('click', function()  { //start function when button is clicked
					jQuery("#save_settings").attr("disabled", true); // Disable button while saving
					jQuery.ajax({
						type: "post",
						url: "admin-ajax.php",
						data: {
							action: 'save_set', // call function "Save settings"
							max_results_per_page: 					escape( jQuery( '#max_results_per_page' ).val() ),

							duplicate_organizers_levenshtein: 		escape( jQuery( '#duplicate_organizers_levenshtein' ).val() ),
							duplicate_venues_levenshtein: 			escape( jQuery( '#duplicate_venues_levenshtein' ).val() ),

							google_places_api_key: 					escape( jQuery( '#google_places_api_key' ).val() ),
							google_places_language: 				escape( jQuery( '#google_places_language' ).val() ),
							dataforseo_keyword:						escape( jQuery( '#dataforseo_keyword' ).val() ),
							dataforseo_api_login:					escape( jQuery( '#dataforseo_api_login' ).val() ),
							dataforseo_api_password:				escape( jQuery( '#dataforseo_api_password' ).val() ),
							dataforseo_location_code:				escape( jQuery( '#dataforseo_location_code' ).val() ),
							dataforseo_language_code:				escape( jQuery( '#dataforseo_language_code' ).val() ),							
							dataforseo_date_range:					escape( jQuery( '#dataforseo_date_range' ).val() ),							
							max_images:								escape( jQuery( '#max_images' ).val() ),		
						
							openai_api_key: 						escape( jQuery( '#openai_api_key' ).val() ),
							openai_model:							escape( jQuery( '#openai_model' ).val() ),							
							openai_max_tokens: 						escape( jQuery( '#openai_max_tokens' ).val() ),									
							openai_language: 						escape( jQuery( '#openai_language' ).val() ),
							openai_prompt_headings: 				document.getElementById( 'openai_prompt_headings' ).checked,										
							openai_prompt_faq: 						document.getElementById( 'openai_prompt_faq' ).checked,	
							openai_prompt_html: 					document.getElementById( 'openai_prompt_html' ).checked,
							openai_replace_content: 				document.getElementById( 'openai_replace_content' ).checked,	
							openai_hide_generate_description_buttons:	document.getElementById( 'openai_hide_generate_description_buttons' ).checked,	

							google_safe_browsing: 					document.getElementById( 'google_safe_browsing' ).checked,
							validate_url_https: 					document.getElementById( 'validate_url_https' ).checked,
							validate_url_redirected_moved: 			document.getElementById( 'validate_url_redirected_moved' ).checked,							

							phone_number_format: 					escape( jQuery( '#phone_number_format' ).val() ),
							address_format: 						escape( jQuery( '#address_format' ).val() ),		
							preselection: 							escape( jQuery( '#preselection' ).val() ),

							element_id: 							escape( jQuery( '#element_id' ).val() ),
							show_ignored_objects: 					document.getElementById( 'show_ignored_objects' ).checked,	
							show_events_added: 						document.getElementById( 'show_events_added' ).checked,
							
							ignored_field_venue_description: 		document.getElementById( 'ignored_field_venue_description' ).checked,			
							ignored_field_venue_address: 			document.getElementById( 'ignored_field_venue_address' ).checked,								
							ignored_field_venue_zip: 				document.getElementById( 'ignored_field_venue_zip' ).checked,								
							ignored_field_venue_city: 				document.getElementById( 'ignored_field_venue_city' ).checked,								
							ignored_field_venue_province: 			document.getElementById( 'ignored_field_venue_province' ).checked,								
							ignored_field_venue_state: 				document.getElementById( 'ignored_field_venue_state' ).checked,								
							ignored_field_venue_country: 			document.getElementById( 'ignored_field_venue_country' ).checked,								
							ignored_field_venue_phone: 				document.getElementById( 'ignored_field_venue_phone' ).checked,								
							ignored_field_venue_website: 			document.getElementById( 'ignored_field_venue_website' ).checked,	
							ignored_field_venue_business_status: 	document.getElementById( 'ignored_field_venue_business_status' ).checked,
																					
							ignored_field_organizer_description: 	document.getElementById( 'ignored_field_organizer_description' ).checked,								
							ignored_field_organizer_phone: 			document.getElementById( 'ignored_field_organizer_phone' ).checked,								
							ignored_field_organizer_website: 		document.getElementById( 'ignored_field_organizer_website' ).checked,								
							ignored_field_organizer_mail: 			document.getElementById( 'ignored_field_organizer_mail' ).checked,
							ignored_field_organizer_business_status:document.getElementById( 'ignored_field_organizer_business_status' ).checked,
																						
							ignored_field_event_description: 		document.getElementById( 'ignored_field_event_description' ).checked,								
							ignored_field_event_start: 				document.getElementById( 'ignored_field_event_start' ).checked,								
							ignored_field_event_end: 				document.getElementById( 'ignored_field_event_end' ).checked,								
							ignored_field_event_venue: 				document.getElementById( 'ignored_field_event_venue' ).checked,								
							ignored_field_event_organizer: 			document.getElementById( 'ignored_field_event_organizer' ).checked,
							ignored_field_event_website: 			document.getElementById( 'ignored_field_event_website' ).checked,								
							ignored_field_event_currency_symbol: 	document.getElementById( 'ignored_field_event_currency_symbol' ).checked,		
							ignored_field_event_cost: 				document.getElementById( 'ignored_field_event_cost' ).checked,								
					
							//remove_settings: 						escape( jQuery( '#remove_settings' ).val() ),																							
							_ajax_nonce: '<?php 
    echo  $nonce ;
    ?>'																		
						},
						beforeSend: function() {
							jQuery("#loading_tab_settings").fadeIn('fast');
							jQuery("#show_tab_settings").fadeOut("slow");
						}, 
						success: function(html){ //if data is retrieved, store it in html
							jQuery("#loading_tab_settings").fadeOut('slow');
							jQuery("#show_tab_settings").html( html ); //show the html inside formstatus div
							jQuery("#show_tab_settings").fadeIn("fast"); //animation
							jQuery("#save_settings").attr("disabled", false);
							location.reload(); // reload setting page that everything is shown with updated data
						}
					}); //close jQuery.ajax
					return false;					
				});

				jQuery('#reset_objects_added_button').on('click', function()  {
					jQuery("#reset_objects_added_button").attr("disabled", true);	
					jQuery.ajax({
						type: "post",
						url: "admin-ajax.php",
						data: { 
							action: 'reset_object_li', // call function "reset objects list"
							status: 'added',
							_ajax_nonce: '<?php 
    echo  $nonce ;
    ?>' 
						},
						beforeSend: function() {
						},
						success: function(html){
							jQuery("#reset_objects_added_button").attr("disabled", false);
							location.reload(); // reload setting page that everything is shown with updated data						
						}
					});
					return false;					
				});	

				jQuery('#reset_objects_ignored_button').on('click', function()  {
					jQuery("#reset_objects_ignored_button").attr("disabled", true);	
					jQuery.ajax({
						type: "post",
						url: "admin-ajax.php",
						data: { 
							action: 'reset_object_li', // call function "reset objects list"
							status: 'ignored',
							_ajax_nonce: '<?php 
    echo  $nonce ;
    ?>' 
						},
						beforeSend: function() {
						},
						success: function(html){
							jQuery("#reset_objects_ignored_button").attr("disabled", false);
							location.reload(); // reload setting page that everything is shown with updated data						
						}
					});
					return false;					
				});	
																			
			});
			
			function remove_object(id) {

				jQuery("#object_"+id).attr('disabled','disabled').css('opacity',0.4);
				jQuery.ajax({
					type: "post",
					url: "admin-ajax.php",
					data: { 
						action: 'remove_obj',
						_ajax_nonce: '<?php 
    echo  $nonce ;
    ?>' ,
						id: id,
					},
					beforeSend: function() {
						jQuery("#object_"+id).fadeOut('fast');	// Fade out status after data is ignored	
					},
					success: function(html) {					
					}
				}); //close jQuery.ajax
				return false;	
			}			

		</script>
		
	<?php 
}

//----------------------------------------------------------------------------------------
// Ajax - helloworld
// https://odd.blog/2008/11/01/make-your-wordpress-plugin-talk-ajax/
//----------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------
// Venues with invalid data
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_venues_wid', 'venues_with_invalid_data' );
function venues_with_invalid_data()
{
    check_ajax_referer( 'events-optimizer-nonce' );
    // Maximum number of results to display per page
    $max_results_per_page = get_option( 'teo_max_results_per_page', 5 );
    // Objects to be ignored while searching for invalid data
    $ignored_and_added_objects = unserialize( get_option( 'teo_ignored_and_added_objects' ) );
    if ( empty($ignored_and_added_objects) ) {
        $ignored_and_added_objects = array();
    }
    // Whether ignored objects should be shown or not
    $show_ignored_objects = get_option( 'teo_show_ignored_objects' );
    // Whether added objects should be shown or not
    $show_added_objects = get_option( 'teo_show_added_objects' );
    // Fields to be ignored while searching for invalid data
    $ignored_field_venue_description = get_option( 'teo_ignored_field_venue_description' );
    $ignored_field_venue_address = get_option( 'teo_ignored_field_venue_address' );
    $ignored_field_venue_zip = get_option( 'teo_ignored_field_venue_zip' );
    $ignored_field_venue_city = get_option( 'teo_ignored_field_venue_city' );
    $ignored_field_venue_province = get_option( 'teo_ignored_field_venue_province' );
    $ignored_field_venue_state = get_option( 'teo_ignored_field_venue_state' );
    $ignored_field_venue_country = get_option( 'teo_ignored_field_venue_country' );
    $ignored_field_venue_phone = get_option( 'teo_ignored_field_venue_phone' );
    $ignored_field_venue_website = get_option( 'teo_ignored_field_venue_website' );
    // Custom countries from Events Calendar Options
    $options = get_option( 'tribe_events_calendar_options', '' );
    
    if ( !empty($options['tribe_events_calendar_options']) ) {
        $countries = explode( PHP_EOL, $options['tribeEventsCountries'] );
    } else {
        $countries = array();
    }
    
    foreach ( $countries as $j => $country ) {
        if ( !empty(trim( substr( $country, strpos( $country, ' ' ) + 1 ) )) ) {
            $countries_arr[$j] = trim( substr( $country, strpos( $country, ' ' ) + 1 ) );
        }
    }
    $fields_arr = array(
        'ID'             => __( 'ID', 'events-optimizer' ),
        'post_title'     => __( 'Name', 'events-optimizer' ),
        'post_content'   => __( 'Description', 'events-optimizer' ),
        '_VenueAddress'  => __( 'Address', 'events-optimizer' ),
        '_VenueZip'      => __( 'ZIP', 'events-optimizer' ),
        '_VenueCity'     => __( 'City', 'events-optimizer' ),
        '_VenueProvince' => __( 'Province', 'events-optimizer' ),
        '_VenueState'    => __( 'State', 'events-optimizer' ),
        '_VenueCountry'  => __( 'Country', 'events-optimizer' ),
        '_VenuePhone'    => __( 'Phone', 'events-optimizer' ),
        '_VenueURL'      => __( 'Website', 'events-optimizer' ),
    );
    // Get the next index of the venue if function is called by "Show more" button
    
    if ( isset( $_POST['next_result'] ) ) {
        $next_result = htmlspecialchars( urldecode( $_POST['next_result'] ) );
    } else {
        $next_result = 0;
    }
    
    $k = $next_result;
    // Set index to the next upcoming venue
    $venue = tribe_get_venues();
    $c = count( $venue );
    // Maximum amount of venues
    $i = 0;
    // Index to count for maximum results per page
    // Check if not all venues have been shown yet and maximum results has not been reached
    while ( $k < $c and $i < $max_results_per_page ) {
        // Hide venues that have no errors
        $found_error = false;
        $venue_details = get_post_meta( $venue[$k]->ID );
        $venue_details = $venue_details + array(
            'ID' => array( $venue[$k]->ID ),
        );
        $venue_details = $venue_details + array(
            'post_title' => array( $venue[$k]->post_title ),
        );
        $venue_details = $venue_details + array(
            'post_content' => array( $venue[$k]->post_content ),
        );
        // If venue isn't part of the ignored or added objects list show it
        
        if ( array_search( $venue[$k]->ID, array_column( $ignored_and_added_objects, 'post_id' ) ) === false or $show_ignored_objects === 'true' or $show_added_objects === 'true' ) {
            echo  '<form action="" method="POST" id="save_venue_wid_form_' . $k . '">' ;
            echo  '<br>' ;
            echo  '<table class="widefat fixed striped"><thead><tr><th>' ;
            \esc_html_e( 'Field', 'events-optimizer' );
            echo  '</th><th>' ;
            \esc_html_e( 'Value', 'events-optimizer' );
            echo  '</th><th>' ;
            \esc_html_e( 'Status', 'events-optimizer' );
            echo  '</th></tr></thead><tbody>' ;
            foreach ( $fields_arr as $field => $fieldname ) {
                // Get value of Events Calendar field
                // It should always an string but somehow sometimes it is an array
                
                if ( isset( $venue_details[$field] ) ) {
                    
                    if ( is_array( $venue_details[$field] ) ) {
                        // Convert the array to a string
                        $value = implode( ', ', $venue_details[$field] );
                    } else {
                        $value = (string) $venue_details[$field];
                    }
                
                } else {
                    $value = '';
                }
                
                switch ( $field ) {
                    case 'ID':
                        echo  '<tr><td>' . $fieldname . '</td><input type="hidden" name="' . $field . '" id="' . $field . '" value="' . $value . '"/><td><a href="/wp-admin/post.php?post=' . $venue[$k]->ID . '&action=edit" target="_blank">' . $value . ' <img src="' . plugin_dir_url( __FILE__ ) . 'assets/images/external-link.png" class="external-link-icon" alt="External Link"/></a></td><td>--</td></tr>' ;
                        break;
                    case 'post_title':
                        
                        if ( empty($value) ) {
                            echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/> <a href="https://www.google.com/search?q=' . urlencode( $venue[$k]->post_title . ' ' . $venue[$k]->_VenueCity ) . '" target="_blank"><img src="' . plugin_dir_url( __FILE__ ) . 'assets/images/search.svg" class="search-icon" alt="Search"/></a></td><td><span class="field-status-error">' . __( 'Field is empty', 'events-optimizer' ) . '</span></td></tr>' ;
                            $found_error = true;
                        } else {
                            echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/> <a href="https://www.google.com/search?q=' . urlencode( $venue[$k]->post_title . ' ' . $venue[$k]->_VenueCity ) . '" target="_blank"><img src="' . plugin_dir_url( __FILE__ ) . 'assets/images/search.svg" class="search-icon" alt="Search"/></a></td><td></td></tr>' ;
                        }
                        
                        break;
                    case 'post_content':
                        
                        if ( empty($value) ) {
                            echo  '<tr><td>' . $fieldname . '</td><td><textarea name="' . $field . '" id="' . $field . '" rows="6" cols="30"></textarea></td><td><span class="field-status-error">' . __( 'Field is empty', 'events-optimizer' ) . '</span></td></tr>' ;
                            if ( $ignored_field_venue_description === 'false' ) {
                                $found_error = true;
                            }
                        } else {
                            echo  '<tr><td>' . $fieldname . '</td><td><textarea name="' . $field . '" id="' . $field . '" rows="6" cols="30">' . esc_textarea( $value ) . '</textarea></td><td><span class="field-status-ok">' . __( 'OK', 'events-optimizer' ) . '</span></td></tr>' ;
                        }
                        
                        break;
                    case '_VenueZip':
                        
                        if ( empty($value) ) {
                            echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/></td><td><span class="field-status-error">' . __( 'Field is empty', 'events-optimizer' ) . '</span></td></tr>' ;
                            if ( $ignored_field_venue_zip === 'false' ) {
                                $found_error = true;
                            }
                        } else {
                            echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/></td><td><span class="field-status-ok">' . __( 'OK', 'events-optimizer' ) . '</span></td></tr>' ;
                        }
                        
                        break;
                        // Only for Non-USA countries: set the province (= Bundesland)
                    // Only for Non-USA countries: set the province (= Bundesland)
                    case '_VenueProvince':
                        if ( isset( $venue_details['_VenueCountry'][0] ) and $venue_details['_VenueCountry'][0] != 'United States' ) {
                            
                            if ( empty($value) ) {
                                echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/></td><td><span class="field-status-error">' . __( 'Field is empty', 'events-optimizer' ) . '</span></td></tr>' ;
                                if ( $ignored_field_venue_province === 'false' ) {
                                    $found_error = true;
                                }
                            } else {
                                echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/></td><td><span class="field-status-ok">' . __( 'OK', 'events-optimizer' ) . '</span></td></tr>' ;
                            }
                        
                        }
                        break;
                        // Only for USA - set abbreviation of state, e.g. AZ for Arizona)
                    // Only for USA - set abbreviation of state, e.g. AZ for Arizona)
                    case '_VenueState':
                        if ( isset( $venue_details['_VenueCountry'][0] ) and $venue_details['_VenueCountry'][0] === 'United States' ) {
                            
                            if ( empty($value) ) {
                                echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/></td><td><span class="field-status-error">' . __( 'Field is empty', 'events-optimizer' ) . '</span></td></tr>' ;
                                if ( $ignored_field_venue_state === 'false' ) {
                                    $found_error = true;
                                }
                            } else {
                                echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/></td><td><span class="field-status-ok">' . __( 'OK', 'events-optimizer' ) . '</span></td></tr>' ;
                            }
                        
                        }
                        break;
                        // Check if country is in custom country list
                    // Check if country is in custom country list
                    case '_VenueCountry':
                        
                        if ( empty($value) ) {
                            echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/></td><td><span class="field-status-error">' . __( 'Field is empty', 'events-optimizer' ) . '</span></td></tr>' ;
                            if ( $ignored_field_venue_country === 'false' ) {
                                $found_error = true;
                            }
                        } else {
                            // Custom countries available?
                            
                            if ( !empty($countries_arr) ) {
                                
                                if ( !in_array( $value, $countries_arr ) ) {
                                    echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/></td><td><span class="field-status-error">' . __( 'Country not in custom country list', 'events-optimizer' ) . '</span></td></tr>' ;
                                    if ( $ignored_field_venue_country === 'false' ) {
                                        $found_error = true;
                                    }
                                } else {
                                    echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/></td><td><span class="field-status-ok">' . __( 'OK', 'events-optimizer' ) . '</span></td></tr>' ;
                                }
                            
                            } else {
                                echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/></td><td><span class="field-status-ok">' . __( 'OK', 'events-optimizer' ) . '</span></td></tr>' ;
                            }
                        
                        }
                        
                        break;
                    case '_VenuePhone':
                        $valid = phone_number_valid( $value );
                        
                        if ( $valid != '' ) {
                            echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/> <a href="https://www.google.com/search?q=' . urlencode( $venue[$k]->post_title . ' ' . $venue[$k]->_VenueCity ) . ' phone" target="_blank"><img src="' . plugin_dir_url( __FILE__ ) . 'assets/images/search.svg" class="search-icon" alt="Search"/></a></td><td><span class="field-status-error">' . $valid . '</td></tr>' ;
                            if ( $ignored_field_venue_phone === 'false' ) {
                                $found_error = true;
                            }
                        } else {
                            echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/> <a href="https://www.google.de/search?q=' . urlencode( $venue[$k]->post_title ) . ' phone" target="_blank"> <img src="' . plugin_dir_url( __FILE__ ) . 'assets/images/search.svg" class="search-icon" alt="Search"/></a></td><td><span class="field-status-ok">' . __( 'OK', 'events-optimizer' ) . '</span></td></tr>' ;
                        }
                        
                        break;
                    case '_VenueURL':
                        $valid = url_valid( $value );
                        
                        if ( $valid != '' ) {
                            echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/> <a href="https://www.google.com/search?q=' . urlencode( $venue[$k]->post_title . ' ' . $venue[$k]->_VenueCity ) . '" target="_blank"><img src="' . plugin_dir_url( __FILE__ ) . 'assets/images/search.svg" class="search-icon" alt="Search"/></a></td><td><span class="field-status-error">' . $valid . '</td></tr>' ;
                            if ( $ignored_field_venue_website === 'false' ) {
                                $found_error = true;
                            }
                        } else {
                            echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/> <a href="' . $value . '" target="_blank"><img src="' . plugin_dir_url( __FILE__ ) . 'assets/images/external-link.png" class="external-link-icon" alt="External Link"/></a></td><td><span class="field-status-ok">' . __( 'OK', 'events-optimizer' ) . '</span></td></tr>' ;
                        }
                        
                        break;
                    default:
                        
                        if ( empty($value) ) {
                            echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/></td><td><span class="field-status-error">' . __( 'Field is empty', 'events-optimizer' ) . '</span></td></tr>' ;
                            if ( $field === 'post_title' ) {
                                $found_error = true;
                            }
                            if ( $field === '_VenueAddress' and $ignored_field_venue_address === 'false' ) {
                                $found_error = true;
                            }
                            if ( $field === '_VenueCity' and $ignored_field_venue_city === 'false' ) {
                                $found_error = true;
                            }
                        } else {
                            echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/></td><td><span class="field-status-ok">' . __( 'OK', 'events-optimizer' ) . '</span></td></tr>' ;
                        }
                
                }
                // end switch
            }
            // end foreach
            echo  '</tbody>
				</table>
				<p><input type="button" name="action" class="button button-primary" id="save_venue_wid_button_' . $k . '" value="' . __( 'Save changes', 'events-optimizer' ) . '" onclick="save_form(' . "'" . $k . "'" . ');" />
				&nbsp;&nbsp;&nbsp;<input type="button" name="action" class="button delete" id="ignore_object_button_' . $k . '" value="' . __( 'Ignore', 'events-optimizer' ) . '" onclick="ignore_object(' . "'" . $k . "'" . ');"' ;
            echo  "disabled" ;
            echo  '/>
				</p><div id="status_save_venue_wid_' . $k . '">' . __( 'SAVING', 'events-optimizer' ) . '</div></p><br>
				</form>' ;
        }
        
        // end if array_search(
        $nonce = wp_create_nonce( 'events-optimizer-nonce' );
        ?>	
		
		<script  type='text/javascript'>
	
			function save_form(form_id){
				var datastring = jQuery("#save_venue_wid_form_"+form_id).serialize();
				jQuery("#save_venue_wid_button_"+form_id).attr("disabled", true); // Disable button while saving
				jQuery.ajax({
					type: "post",
					url: "admin-ajax.php",
					data: { 
						action: 'save_ven',
						_ajax_nonce: '<?php 
        echo  $nonce ;
        ?>' ,
						data: datastring,
					},
					beforeSend: function() {
						jQuery("#status_save_venue_wid_"+form_id).fadeIn('fast');  // Fade in status when button is clicked
					},
					success: function(html) {
						jQuery("#status_save_venue_wid_"+form_id).fadeOut('slow');	// Fade out status after data is saved
						jQuery("#save_venue_wid_button_"+form_id).attr("disabled", false); // Enable button after saving
						jQuery("#save_venue_wid_form_"+form_id).slideUp(); // Disable venue form after saving							
					}
				}); //close jQuery.ajax
				return false;	
			}
			function ignore_object(form_id){	
				var datastring = jQuery("#save_venue_wid_form_"+form_id).serialize() + '&type=tribe_venue&status=ignored';
				jQuery("#ignore_object_button_"+form_id).attr("disabled", true); // Disable button while ignoring
				jQuery.ajax({
					type: "post",
					url: "admin-ajax.php",
					data: { 
						action: 'add_obj',
						_ajax_nonce: '<?php 
        echo  $nonce ;
        ?>' ,
						data: datastring,
					},
					beforeSend: function() {
					},
					success: function(html) {
						jQuery("#status_save_venue_wid_"+form_id).fadeOut('slow');	// Fade out status after data is ignored
						jQuery("#ignore_object_button_"+form_id).attr("disabled", false); // Enable button after ignoring
						jQuery("#save_venue_wid_form_"+form_id).slideUp(); // Slide venue form away after ignoring							
					}
				}); //close jQuery.ajax
				return false;	
			}
		</script>	

		<style type='text/css'>
			#status_save_venue_wid_<?php 
        echo  $k ;
        ?> { 
				display: none;
				clear: both;
				background: url(<?php 
        echo  plugin_dir_url( __FILE__ ) . 'assets/images/loading.gif' ;
        ?>) left top no-repeat; 
				text-align: left;
				padding: 0px 0px 30px 25px; 
			}
			<?php 
        if ( $found_error === false ) {
            echo  "#save_venue_wid_form_" . $k . ", #save_venue_wid_button_" . $k . " { display: none; }" ;
        }
        ?>
		</style>

		<?php 
        // if invalid data is found increase counter for maximum results per page
        if ( $found_error === true ) {
            $i++;
        }
        $k++;
    }
    // end while
    
    if ( $k < $c ) {
        echo  '<div id="additional_results_venues_wid_' . $k . '"><p><strong>' . __( 'Additional results available.', 'events-optimizer' ) . '</strong></p></div>' ;
        echo  '<p><input type="submit" name="action" class="button button-primary" id="show_more_venues_wid_button_' . $k . '" value="' . __( 'Show more', 'events-optimizer' ) . '" onclick="show_more(' . "'" . $k . "'" . ');" /></p>' ;
        echo  '<div id="show_more_venues_wid"></div>' ;
        echo  '<div id="status_show_more_venues_wid_' . $k . '">' . __( 'ANALYZING', 'events-optimizer' ) . '</div>' ;
    }
    
    if ( $next_result + $max_results_per_page >= $c ) {
        echo  '<span class="field-status-ok">' . __( 'No (more) results found!', 'events-optimizer' ) . '</span>' ;
    }
    $nonce = wp_create_nonce( 'events-optimizer-nonce' );
    ?>	
		
	<script  type='text/javascript'>
	
		function show_more(form_id){
       		jQuery("#show_more_venues_wid_button_"+form_id).fadeOut('slow'); // Hide button after  		
    		jQuery("#show_more_venues_wid_button_"+form_id).attr("disabled", true); // Disable button while retrieving more results
    		jQuery.ajax({
				type: "post",
				url: "admin-ajax.php",
				data: { 
					action: 'venues_wid',
					next_result: form_id,
					_ajax_nonce: '<?php 
    echo  $nonce ;
    ?>' 
				},
				beforeSend: function() {
					jQuery("#status_show_more_venues_wid_"+form_id).fadeIn('fast');  // Fade in status when button is clicked
					jQuery("#additional_results_venues_wid_"+form_id).fadeOut('slow');	// Fade out text "Additional results ..." after data is saved
				},
				success: function(html) {
					jQuery("#show_more_venues_wid_button_"+form_id).attr("disabled", false); // Enable button after
					jQuery("#status_show_more_venues_wid_"+form_id).fadeOut('slow');	// Fade out status after data is saved
					jQuery("#show_more_venues_wid").append(html); // Add the additional results to page
				}
    		}); //close jQuery.ajax
    		return false;	
    	}	
	</script>

	<style type='text/css'>
		#status_show_more_venues_wid_<?php 
    echo  $k ;
    ?> { 
			display: none;
			clear: both;
			background: url(<?php 
    echo  plugin_dir_url( __FILE__ ) . 'assets/images/loading.gif' ;
    ?>) left top no-repeat; 
			text-align: left;
			padding: 0px 0px 30px 25px; 
		}
	</style>

	<?php 
    die;
}

//----------------------------------------------------------------------------------------
// Save venue data (for invalid data and Google Places optimization)
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_save_ven', 'save_venue' );
function save_venue()
{
    check_ajax_referer( 'events-optimizer-nonce' );
    parse_str( $_POST['data'], $venue );
    // Title and description can't be updated directly with the TEC API (empty fields gets ignored - bug?) and therefore it needs to be updated seperatly
    $venue_post_arr = array(
        'ID'           => sanitize_text_field( $venue['ID'] ),
        'post_title'   => sanitize_text_field( $venue['post_title'] ),
        'post_content' => wp_kses_post( $venue['post_content'] ),
    );
    wp_update_post( $venue_post_arr );
    // Check if state and province are set
    if ( !isset( $venue['_VenueState'] ) ) {
        $venue['_VenueState'] = '';
    }
    if ( !isset( $venue['_VenueProvince'] ) ) {
        $venue['_VenueProvince'] = '';
    }
    // Update venues with values
    $venue_arr = array(
        'Address'       => sanitize_text_field( $venue['_VenueAddress'] ),
        'GeoAddress'    => sanitize_text_field( $venue['_VenueAddress'] ) . ' ' . sanitize_text_field( $venue['_VenueZip'] ) . ' ' . sanitize_text_field( $venue['_VenueCity'] ) . ' ' . sanitize_text_field( $venue['_VenueCountry'] ),
        'Zip'           => sanitize_text_field( $venue['_VenueZip'] ),
        'City'          => sanitize_text_field( $venue['_VenueCity'] ),
        'State'         => sanitize_text_field( $venue['_VenueState'] ),
        'Province'      => sanitize_text_field( $venue['_VenueProvince'] ),
        'StateProvince' => sanitize_text_field( $venue['_VenueProvince'] ),
        'Country'       => sanitize_text_field( $venue['_VenueCountry'] ),
        'Phone'         => sanitize_text_field( $venue['_VenuePhone'] ),
        'URL'           => sanitize_text_field( $venue['_VenueURL'] ),
    );
    tribe_update_venue( $venue['ID'], $venue_arr );
    die;
}

//----------------------------------------------------------------------------------------
// Duplicate Venues
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_duplicate_ven', 'get_duplicate_venues' );
function get_duplicate_venues()
{
}

//----------------------------------------------------------------------------------------
// Merge Venues
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_merge_ven', 'merge_venues' );
function merge_venues()
{
}

//----------------------------------------------------------------------------------------
// Venues: Optimization with Google Places
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_venues_optimization_wgp', 'venues_optimization_with_google_places' );
function venues_optimization_with_google_places()
{
}

//----------------------------------------------------------------------------------------
// Generate description for single venue
//----------------------------------------------------------------------------------------
function generate_venue_description()
{
    // Activate only for Pro version
    
    if ( teo_fs()->can_use_premium_code__premium_only() and get_option( 'teo_openai_hide_generate_description_buttons' ) === 'false' ) {
        $nonce = wp_create_nonce( 'events-optimizer-nonce' );
        wp_enqueue_style( 'style.css', plugin_dir_url( __FILE__ ) . 'assets/css/style.css' );
        // Add button to existing meta box inside organizer form
        ?>	
	
		<input type="button" name="action" id="generate_venue_description_button" class="button-primary" value="<?php 
        \esc_attr_e( 'Generate description with OpenAI', 'events-optimizer' );
        ?>" />
		<span class="field-tip">
			&nbsp;<span class="question-mark-centered"><img src ="<?php 
        echo  plugin_dir_url( __FILE__ ) . 'assets/images/question-mark.png' ;
        ?>"></span>
			<span class="tip-content"><?php 
        echo  __( 'Venue name and city are used to generate the description. Please enter at least the name!', 'events-optimizer' ) ;
        ?></span>
		</span>

		<script type='text/javascript'>

			jQuery(document).ready(function() {

				jQuery("#generate_venue_description_button").on('click', function()  {

					jQuery("#generate_venue_description_button").attr("disabled", true); // Disable button while retrieving more results
					
					// Get existing content from TinyMCE editor or text editor
					if (typeof tinyMCE !== "undefined" && tinyMCE.activeEditor != null && !tinyMCE.activeEditor.isHidden()) {
 	
      					existing_content = tinymce.activeEditor.getContent();
					} else if ( jQuery('#content').length > 0 ) { 
						existing_content = jQuery('#content').val();
					}
					
					searchstring = 	document.getElementById('title').value + " " +
									document.getElementById('venueCity').value;

					if ( searchstring.trim() === "" || document.getElementById("title").value === "" ) { 
    					alert( 'Venue name and city are used to generate the description. Please enter at least the name!', 'events_optimizer' );
   						jQuery("#generate_venue_description_button").attr("disabled", false);// Enable button after	
					} else {							
					
						jQuery.ajax({
							type: "POST",
							url: "admin-ajax.php",
							data: { 		
								action: 'generate_venue_description_with_oai', // call function "generate_venue_description_with_oai"					
								_ajax_nonce: '<?php 
        echo  $nonce ;
        ?>',
								searchstring: searchstring,
								existing_content: existing_content
							},
							beforeSend: function() {
								jQuery("body").css("cursor", "progress");						
							},
							success : function( data ) {

								info = JSON.parse( data );
								
								// Error handling
								if ( info.error ) {
									alert( info.error );
								} else {

									// Check if info.description is defined and not empty
									if (info.description && info.description.trim() !== "") {
										// Add generated description to content field (either TinymCE or text editor)
										if (typeof tinyMCE !== "undefined" && tinyMCE.activeEditor != null && !tinyMCE.activeEditor.isHidden()) {
											tinyMCE.activeEditor.setContent(info.description);
										} else if (jQuery("#content").length > 0) {
											jQuery("#content").val(info.description);
										} else {
											console.error("Unable to set content: No suitable editor or text area found");
										}
									} else {
										console.warn("Description is empty or undefined");
									}
								}
								
								jQuery("#generate_venue_description_button").attr("disabled", false); // Enable button after
								jQuery("body").css("cursor", "default");
							}
						})
					}
					return false;					
				});											
			});
			
		</script>
	
		<?php 
    }

}

//----------------------------------------------------------------------------------------
// Generate venue description and hand it over to Java Script function above
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_generate_venue_description_with_oai', 'generate_venue_description_with_open_ai' );
function generate_venue_description_with_open_ai()
{
}

//----------------------------------------------------------------------------------------
// Fetch data from Google Places for single venue
//----------------------------------------------------------------------------------------
add_action( 'tribe_events_venue_before_metabox', 'fetch_venue_data' );
function fetch_venue_data()
{
}

//----------------------------------------------------------------------------------------
// Get venue data from Google Places and hand it over to Java Script function above
// https://www.codexworld.com/post-get-json-data-from-php-script-jquery-ajax/
// https://www.mediaevent.de/javascript/json.html
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_fetch_venue_data_fgp', 'fetch_venue_data_from_google_places' );
function fetch_venue_data_from_google_places()
{
}

//----------------------------------------------------------------------------------------
// Organizers with invalid data
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_organizers_wid', 'organizers_with_invalid_data' );
function organizers_with_invalid_data()
{
    check_ajax_referer( 'events-optimizer-nonce' );
    // Maximum number of results to display per page
    $max_results_per_page = get_option( 'teo_max_results_per_page', 5 );
    // Objects to be ignored while searching for invalid data
    $ignored_and_added_objects = unserialize( get_option( 'teo_ignored_and_added_objects' ) );
    if ( empty($ignored_and_added_objects) ) {
        $ignored_and_added_objects = array();
    }
    // Whether ignored objects should be shown or not
    $show_ignored_objects = get_option( 'teo_show_ignored_objects' );
    // Whether added objects should be shown or not
    $show_added_objects = get_option( 'teo_show_added_objects' );
    // Fields to be ignored while searching for invalid data
    $ignored_field_organizer_description = get_option( 'teo_ignored_field_organizer_description' );
    $ignored_field_organizer_phone = get_option( 'teo_ignored_field_organizer_phone' );
    $ignored_field_organizer_website = get_option( 'teo_ignored_field_organizer_website' );
    $ignored_field_organizer_mail = get_option( 'teo_ignored_field_organizer_mail' );
    $fields_arr = array(
        'ID'                => __( 'ID', 'events-optimizer' ),
        'post_title'        => __( 'Name', 'events-optimizer' ),
        'post_content'      => __( 'Description', 'events-optimizer' ),
        '_OrganizerPhone'   => __( 'Phone', 'events-optimizer' ),
        '_OrganizerWebsite' => __( 'Website', 'events-optimizer' ),
        '_OrganizerEmail'   => __( 'Mail', 'events-optimizer' ),
    );
    // Get the next index of the organizer if function is called by "Show more" button
    
    if ( isset( $_POST['next_result'] ) ) {
        $next_result = urldecode( $_POST['next_result'] );
    } else {
        $next_result = 0;
    }
    
    $k = $next_result;
    // Set index to the next upcoming organizer
    $organizer = tribe_get_organizers();
    $c = count( $organizer );
    // Maximum amount of organizers
    $i = 0;
    // Index to count for maximum results per page
    // Check if not all organizers have been shown yet and maximum results has not been reached
    while ( $k < $c and $i < $max_results_per_page ) {
        // Hide organizers that have no errors
        $found_error = false;
        $organizer_details = get_post_meta( $organizer[$k]->ID );
        $organizer_details = $organizer_details + array(
            'ID' => array( $organizer[$k]->ID ),
        );
        $organizer_details = $organizer_details + array(
            'post_title' => array( $organizer[$k]->post_title ),
        );
        $organizer_details = $organizer_details + array(
            'post_content' => array( $organizer[$k]->post_content ),
        );
        // If organizer isn't part of the ignored or added objects list show it
        
        if ( array_search( $organizer[$k]->ID, array_column( $ignored_and_added_objects, 'post_id' ) ) === false or $show_ignored_objects === 'true' or $show_added_objects === 'true' ) {
            echo  '<form action="" method="POST" id="save_organizer_wid_form_' . $k . '">' ;
            echo  '<br>' ;
            echo  '<table class="widefat fixed striped"><thead><tr><th>' ;
            \esc_html_e( 'Field', 'events-optimizer' );
            echo  '</th><th>' ;
            \esc_html_e( 'Value', 'events-optimizer' );
            echo  '</th><th>' ;
            \esc_html_e( 'Status', 'events-optimizer' );
            echo  '</th></tr></thead><tbody>' ;
            foreach ( $fields_arr as $field => $fieldname ) {
                
                if ( isset( $organizer_details[$field][0] ) ) {
                    $value = $organizer_details[$field][0];
                } else {
                    $value = '';
                }
                
                echo  '<input type="hidden" name="index" id="index" value="' . $k . '"/>' ;
                switch ( $field ) {
                    case 'ID':
                        echo  '<tr><td>' . $fieldname . '</td><input type="hidden" name="' . $field . '" id="' . $field . '" value="' . $value . '"/><td><a href="/wp-admin/post.php?post=' . $organizer[$k]->ID . '&action=edit" target="_blank">' . $value . ' <img src="' . plugin_dir_url( __FILE__ ) . 'assets/images/external-link.png" class="external-link-icon" alt="External Link"/></a></td><td>--</td></tr>' ;
                        break;
                    case 'post_title':
                        
                        if ( empty($value) ) {
                            echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/> <a href="https://www.google.com/search?q=' . urlencode( $organizer[$k]->post_title ) . '" target="_blank"><img src="' . plugin_dir_url( __FILE__ ) . 'assets/images/search.svg" class="search-icon" alt="Search"/></a></td><td><span class="field-status-error">' . __( 'Field is empty', 'events-optimizer' ) . '</span></td></tr>' ;
                        } else {
                            echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/> <a href="https://www.google.com/search?q=' . urlencode( $organizer[$k]->post_title ) . '" target="_blank"><img src="' . plugin_dir_url( __FILE__ ) . 'assets/images/search.svg" class="search-icon" alt="Search"/></a></td><td><span class="field-status-ok">' . __( 'OK', 'events-optimizer' ) . '</span></td></tr>' ;
                        }
                        
                        break;
                    case 'post_content':
                        
                        if ( empty($value) ) {
                            echo  '<tr><td>' . $fieldname . '</td><td><textarea name="' . $field . '" id="' . $field . '" rows="6" cols="30"></textarea></td><td><span class="field-status-error">' . __( 'Field is empty', 'events-optimizer' ) . '</span></td></tr>' ;
                            if ( $ignored_field_organizer_description === 'false' ) {
                                $found_error = true;
                            }
                        } else {
                            echo  '<tr><td>' . $fieldname . '</td><td><textarea name="' . $field . '" id="' . $field . '" rows="6" cols="30">' . esc_textarea( $value ) . '</textarea></td><td><span class="field-status-ok">' . __( 'OK', 'events-optimizer' ) . '</span></td></tr>' ;
                        }
                        
                        break;
                    case '_OrganizerPhone':
                        $valid = phone_number_valid( $value );
                        
                        if ( $valid != '' ) {
                            echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/> <a href="https://www.google.com/search?q=' . urlencode( $organizer[$k]->post_title . ' phone' ) . '" target="_blank"><img src="' . plugin_dir_url( __FILE__ ) . 'assets/images/search.svg" class="search-icon" alt="Search"/></a></td><td><span class="field-status-error">' . $valid . '</td></tr>' ;
                            if ( $ignored_field_organizer_phone === 'false' ) {
                                $found_error = true;
                            }
                        } else {
                            echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/> <a href="https://www.google.de/search?q=' . urlencode( $organizer[$k]->post_title ) . ' phone" target="_blank"><img src="' . plugin_dir_url( __FILE__ ) . 'assets/images/search.svg" class="search-icon" alt="Search"/></a></td><td><span class="field-status-ok">' . __( 'OK', 'events-optimizer' ) . '</span></td></tr>' ;
                        }
                        
                        break;
                    case '_OrganizerWebsite':
                        $valid = url_valid( $value );
                        
                        if ( $valid != '' ) {
                            echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/> <a href="https://www.google.com/search?q=' . urlencode( $organizer[$k]->post_title ) . '" target="_blank"><img src="' . plugin_dir_url( __FILE__ ) . 'assets/images/search.svg" class="search-icon" alt="Search"/></a></td><td><span class="field-status-error">' . $valid . '</td></tr>' ;
                            if ( $ignored_field_organizer_website === 'false' ) {
                                $found_error = true;
                            }
                        } else {
                            echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/> <a href="' . $value . '" target="_blank"> <img src="' . plugin_dir_url( __FILE__ ) . 'assets/images/external-link.png" class="external-link-icon" alt="External Link"/></a></td><td><span class="field-status-ok">' . __( 'OK', 'events-optimizer' ) . '</span></td></tr>' ;
                        }
                        
                        break;
                        // Check if mail is valid
                    // Check if mail is valid
                    case '_OrganizerEmail':
                        $valid = mail_valid( $value );
                        
                        if ( $valid != '' ) {
                            echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/> <a href="https://www.google.de/search?q=' . urlencode( $organizer[$k]->post_title . ' mail' ) . '" target="_blank"><img src="' . plugin_dir_url( __FILE__ ) . 'assets/images/search.svg" class="search-icon" alt="Search"/></a></td><td><span class="field-status-error">' . $valid . '</td></tr>' ;
                            if ( $ignored_field_organizer_mail === 'false' ) {
                                $found_error = true;
                            }
                        } else {
                            echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/> <a href="https://www.google.de/search?q=' . urlencode( $organizer[$k]->post_title . ' mail' ) . '" target="_blank"><img src="' . plugin_dir_url( __FILE__ ) . 'assets/images/search.svg" class="search-icon" alt="Search"/></a></td><td><span class="field-status-ok">' . __( 'OK', 'events-optimizer' ) . '</span></td></tr>' ;
                        }
                        
                        break;
                    default:
                        
                        if ( empty($value) ) {
                            echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/> <a href="https://www.google.com/search?q=' . urlencode( $organizer[$k]->post_title ) . '" target="_blank"><img src="' . plugin_dir_url( __FILE__ ) . 'assets/images/search.svg" class="search-icon" alt="Search"/></a></td><td><span class="field-status-error">' . __( 'Field is empty', 'events-optimizer' ) . '</span></td></tr>' ;
                        } else {
                            echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/></td><td><span class="field-status-ok">' . __( 'OK', 'events-optimizer' ) . '</span></td></tr>' ;
                        }
                
                }
                // end switch
            }
            // end foreach
            echo  '</tbody>
				</table>	
				<p><input type="button" name="action" class="button button-primary" id="save_organizer_wid_button_' . $k . '" value="' . __( 'Save changes', 'events-optimizer' ) . '" onclick="save_form(' . "'" . $k . "'" . ');" />
				&nbsp;&nbsp;&nbsp;<input type="button" name="action" class="button delete" id="ignore_object_button_' . $k . '" value="' . __( 'Ignore', 'events-optimizer' ) . '" onclick="ignore_object(' . "'" . $k . "'" . ');"' ;
            echo  "disabled" ;
            echo  '/>
				</p><div id="status_save_organizer_wid_' . $k . '">' . __( 'SAVING', 'events-optimizer' ) . '</div></p><br>
				</form>' ;
        }
        
        // end if array_search(
        $nonce = wp_create_nonce( 'events-optimizer-nonce' );
        ?>	

		<script  type='text/javascript'>
	
			function save_form(form_id){
				var datastring = jQuery("#save_organizer_wid_form_"+form_id).serialize(); 
				jQuery("#save_organizer_wid_button_"+form_id).attr("disabled", true); // Disable button while saving
				jQuery.ajax({
					type: "post",
					url: "admin-ajax.php",
					data: { 
						action: 'save_org',
						_ajax_nonce: '<?php 
        echo  $nonce ;
        ?>' ,
						data: datastring,
						},
					beforeSend: function() {
						jQuery("#status_save_organizer_wid_"+form_id).fadeIn('fast');  // Fade in status when button is clicked
						},
					success: function(html) {
						jQuery("#status_save_organizer_wid_"+form_id).fadeOut('slow');	// Fade out status after data is saved
						jQuery("#save_organizer_wid_button_"+form_id).attr("disabled", false); // Enable button after saving
						jQuery("#save_organizer_wid_form_"+form_id).slideUp(); // Slide organizer form away after saving				
					}
				}); //close jQuery.ajax
				return false;	
			}
			function ignore_object(form_id){
				var datastring = jQuery("#save_organizer_wid_form_"+form_id).serialize() + '&type=tribe_organizer&status=ignored';
				jQuery("#ignore_object_button_"+form_id).attr("disabled", true); // Disable button while ignoring
				jQuery.ajax({
					type: "post",
					url: "admin-ajax.php",
					data: { 
						action: 'add_obj',
						_ajax_nonce: '<?php 
        echo  $nonce ;
        ?>' ,
						data: datastring,
					},
					beforeSend: function() {
					},
					success: function(html) {
						jQuery("#status_save_organizer_wid_"+form_id).fadeOut('slow');	// Fade out status after data is ignored
						jQuery("#ignore_object_button_"+form_id).attr("disabled", false); // Enable button after ignoring
						jQuery("#save_organizer_wid_form_"+form_id).slideUp(); // Slide organizer form away after ignoring							
					}
				}); //close jQuery.ajax
				return false;	
			}
		</script>	
		
		<style type='text/css'>
			#status_save_organizer_wid_<?php 
        echo  $k ;
        ?> { 
				display: none;
				clear: both;
				background: url(<?php 
        echo  plugin_dir_url( __FILE__ ) . 'assets/images/loading.gif' ;
        ?>) left top no-repeat; 
				text-align: left;
				padding: 0px 0px 30px 25px; 
			}
			<?php 
        if ( $found_error === false ) {
            echo  "#save_organizer_wid_form_" . $k . ", #save_organizer_wid_button_" . $k . " { display: none; }" ;
        }
        ?>
		</style>

		<?php 
        // if invalid data is found increase counter for maximum results per page
        if ( $found_error === true ) {
            $i++;
        }
        $k++;
    }
    // end while
    
    if ( $k < $c ) {
        echo  '<div id="additional_results_organizers_wid_' . $k . '"><p><strong>' . __( 'Additional results available.', 'events-optimizer' ) . '</strong></p></div>' ;
        echo  '<p><input type="submit" name="action" class="button button-primary" id="show_more_organizers_wid_button_' . $k . '" value="' . __( 'Show more', 'events-optimizer' ) . '" onclick="show_more(' . "'" . $k . "'" . ');" /></p>' ;
        echo  '<div id="show_more_organizers_wid"></div>' ;
        echo  '<div id="status_show_more_organizers_wid_' . $k . '">' . __( 'ANALYZING', 'events-optimizer' ) . '</div>' ;
    }
    
    if ( $next_result + $max_results_per_page >= $c ) {
        echo  '<span class="field-status-ok">' . __( 'No (more) results found!', 'events-optimizer' ) . '</span>' ;
    }
    $nonce = wp_create_nonce( 'events-optimizer-nonce' );
    ?>	
		
	<script  type='text/javascript'>
	
		function show_more(form_id){
			jQuery("#show_more_organizers_wid_button_"+form_id).fadeOut('slow'); // Hide button after  		
			jQuery("#show_more_organizers_wid_button_"+form_id).attr("disabled", true); // Disable button while retrieving more results
			jQuery.ajax({
				type: "post",
				url: "admin-ajax.php",
				data: { 
					action: 'organizers_wid',
					next_result: form_id,
					_ajax_nonce: '<?php 
    echo  $nonce ;
    ?>' 
				},
				beforeSend: function() {
					jQuery("#status_show_more_organizers_wid_"+form_id).fadeIn('fast');  // Fade in status when button is clicked
					jQuery("#additional_results_organizers_wid_"+form_id).fadeOut('slow');	// Fade out text "Additional results ..." after data is saved
				},
				success: function(html) {
					jQuery("#show_more_organizers_button_"+form_id).attr("disabled", false); // Enable button after
					jQuery("#status_show_more_organizers_wid_"+form_id).fadeOut('slow');	// Fade out status after data is saved
					jQuery("#show_more_organizers_wid").append(html); // Add the additional results to page
				}
			}); //close jQuery.ajax
			return false;	
		}	
	</script>

	<style type='text/css'>
		#status_show_more_organizers_wid_<?php 
    echo  $k ;
    ?> { 
			display: none;
			clear: both;
			background: url(<?php 
    echo  plugin_dir_url( __FILE__ ) . 'assets/images/loading.gif' ;
    ?>) left top no-repeat; 
			text-align: left;
			padding: 0px 0px 30px 25px; 
		}
	</style>

	<?php 
    die;
}

//----------------------------------------------------------------------------------------
// Save organizer data (for invalid data and Optimization with Google Places)
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_save_org', 'save_organizer' );
function save_organizer()
{
    check_ajax_referer( 'events-optimizer-nonce' );
    parse_str( $_POST['data'], $organizer );
    // Title and description are not part of the post_meta therefore they need to be updated seperatly
    $organizer_post_arr = array(
        'ID'           => sanitize_text_field( $organizer['ID'] ),
        'post_title'   => sanitize_text_field( $organizer['post_title'] ),
        'post_content' => wp_kses_post( $organizer['post_content'] ),
    );
    wp_update_post( $organizer_post_arr );
    // Update organizer with values
    $organizer_arr = array(
        'Phone'   => sanitize_text_field( $organizer['_OrganizerPhone'] ),
        'Website' => sanitize_text_field( $organizer['_OrganizerWebsite'] ),
        'Email'   => sanitize_email( $organizer['_OrganizerEmail'] ),
    );
    tribe_update_organizer( $organizer['ID'], $organizer_arr );
    die;
}

//----------------------------------------------------------------------------------------
// Duplicate Organizers
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_duplicate_org', 'duplicate_organizers' );
function duplicate_organizers()
{
}

//----------------------------------------------------------------------------------------
// Merge Organizers
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_merge_org', 'merge_organizers' );
function merge_organizers()
{
}

//----------------------------------------------------------------------------------------
// Organizers: Optimization with Google Places
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_organizers_optimization_wgp', 'organizers_optimization_with_google_places' );
function organizers_optimization_with_google_places()
{
}

//----------------------------------------------------------------------------------------
// Generate description for single organizer
//----------------------------------------------------------------------------------------
function generate_organizer_description()
{
    // Activate only for Pro version
    
    if ( teo_fs()->can_use_premium_code__premium_only() and get_option( 'teo_openai_hide_generate_description_buttons' ) === 'false' ) {
        $nonce = wp_create_nonce( 'events-optimizer-nonce' );
        wp_enqueue_style( 'style.css', plugin_dir_url( __FILE__ ) . 'assets/css/style.css' );
        // Add button to existing meta box inside organizer form
        ?>	
		
		<input type="button" name="action" id="generate_organizer_description_button" class="button-primary" value="<?php 
        \esc_attr_e( 'Generate description with OpenAI', 'events-optimizer' );
        ?>" />
		<span class="field-tip">
			&nbsp;<span class="question-mark-centered"><img src ="<?php 
        echo  plugin_dir_url( __FILE__ ) . 'assets/images/question-mark.png' ;
        ?>"></span>
			<span class="tip-content"><?php 
        echo  __( 'Organizer name is used to generate description. Please enter it (including the city) in the title field!', 'events-optimizer' ) ;
        ?></span>
		</span>


		<script type='text/javascript'>

			jQuery(document).ready(function() {

				jQuery("#generate_organizer_description_button").on('click', function()  {

					jQuery("#generate_organizer_description_button").attr("disabled", true); // Disable button while retrieving more results
					
					// Get existing content from TinyMCE editor or text editor
					if (typeof tinyMCE !== "undefined" && tinyMCE.activeEditor != null && !tinyMCE.activeEditor.isHidden()) {
      					existing_content = tinymce.activeEditor.getContent();
					} else if ( jQuery('#content').length > 0 ) { 
						existing_content = jQuery('#content').val();
					}

					searchstring = 	document.getElementById("title").value;
					
					if ( searchstring.trim() === "" || document.getElementById("title").value === "" ) { 
						alert ("Organizer name is used to search. Please enter it (including the city) in the title field!");
						jQuery("#generate_organizer_description_button").attr("disabled", false); // Enable button after		
					
					} else {							
					
						jQuery.ajax({
							type: "POST",
							url: "admin-ajax.php",
							data: { 		
								action: 'generate_organizer_description_with_oai', // call function "generate_organizer_description_with_oai"					
								_ajax_nonce: '<?php 
        echo  $nonce ;
        ?>',
								searchstring: searchstring,
								existing_content: existing_content
							},
							beforeSend: function() {
								jQuery("body").css("cursor", "progress");						
							},
							success : function( data ) {
						
								info = JSON.parse( data );

								// Error handling
								if ( info.error ) {
									alert( info.error );
								} else {
									// Check if info.description is defined and not empty
									if (info.description && info.description.trim() !== "") {
										// Add generated description to content field (either TinymCE or text editor)
										if (typeof tinyMCE !== "undefined" && tinyMCE.activeEditor != null && !tinyMCE.activeEditor.isHidden()) {
											tinyMCE.activeEditor.setContent(info.description);
										} else if (jQuery("#content").length > 0) {
											jQuery("#content").val(info.description);
										} else {
											console.error("Unable to set content: No suitable editor or text area found");
										}
									} else {
										console.warn("Description is empty or undefined");
									}
								}
								
								jQuery("#generate_organizer_description_button").attr("disabled", false); // Enable button after
								jQuery("body").css("cursor", "default");
							}
						})
					}
					return false;					
				});											
			});
			
		</script>
	
		<?php 
    }

}

//----------------------------------------------------------------------------------------
// Generate organizer description and hand it over to Java Script function above
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_generate_organizer_description_with_oai', 'generate_organizer_description_with_open_ai' );
function generate_organizer_description_with_open_ai()
{
}

//----------------------------------------------------------------------------------------
// Fetch data from Google Places for single organizer
//----------------------------------------------------------------------------------------
add_action( 'tribe_events_organizer_before_metabox', 'fetch_organizer_data' );
function fetch_organizer_data()
{
}

//----------------------------------------------------------------------------------------
// Get organizer data from Google Places and hand it over to Java Script function above
// https://www.codexworld.com/post-get-json-data-from-php-script-jquery-ajax/
// https://www.mediaevent.de/javascript/json.html
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_fetch_organizer_data_from_google_places', 'fetch_organizer_data_from_google_places' );
function fetch_organizer_data_from_google_places()
{
}

//----------------------------------------------------------------------------------------
// Events with invalid data
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_events_wid', 'events_with_invalid_data' );
function events_with_invalid_data()
{
    check_ajax_referer( 'events-optimizer-nonce' );
    // Maximum number of results to display per page
    $max_results_per_page = get_option( 'teo_max_results_per_page', 5 );
    // Objects to be ignored while searching for invalid data
    $ignored_and_added_objects = unserialize( get_option( 'teo_ignored_and_added_objects' ) );
    if ( empty($ignored_and_added_objects) ) {
        $ignored_and_added_objects = array();
    }
    // Whether ignored objects should be shown or not
    $show_ignored_objects = get_option( 'teo_show_ignored_objects' );
    // Whether added objects should be shown or not
    $show_added_objects = get_option( 'teo_show_added_objects' );
    // Fields to be ignored while searching for invalid data
    $ignored_field_event_description = get_option( 'teo_ignored_field_event_description' );
    $ignored_field_event_start = get_option( 'teo_ignored_field_event_start' );
    $ignored_field_event_end = get_option( 'teo_ignored_field_event_end' );
    $ignored_field_event_venue = get_option( 'teo_ignored_field_event_venue' );
    $ignored_field_event_organizer = get_option( 'teo_ignored_field_event_organizer' );
    $ignored_field_event_website = get_option( 'teo_ignored_field_event_website' );
    $ignored_field_event_currency_symbol = get_option( 'teo_ignored_field_event_currency_symbol' );
    $ignored_field_event_cost = get_option( 'teo_ignored_field_event_cost' );
    // Get custom currency symbols from Events Calendar Options
    $options = get_option( 'tribe_events_calendar_options', '' );
    
    if ( !empty($options['tribe_ext_ce_cost_currency_symbols']) ) {
        $currency_symbols_arr = explode( '|', $options['tribe_ext_ce_cost_currency_symbols'] );
    } else {
        $currency_symbols_arr = array();
    }
    
    // If there are no custom currencies use the standard currency from the Events Calendar Options
    if ( empty(array_filter( $currency_symbols_arr )) ) {
        if ( !empty($options['defaultCurrencySymbol']) ) {
            $currency_symbols_arr[0] = $options['defaultCurrencySymbol'];
        }
    }
    // Get existing venues from Events Calendar and create sorted array for dropdown list
    $venues_unordered_arr = tribe_get_venues();
    foreach ( $venues_unordered_arr as $venue ) {
        $venues_arr[] = array(
            'ID'   => $venue->ID,
            'Name' => $venue->post_title,
        );
    }
    setlocale( LC_COLLATE, get_locale() );
    $name_arr = array_column( $venues_arr, 'Name' );
    array_multisort(
        $name_arr,
        SORT_ASC,
        SORT_LOCALE_STRING,
        $venues_arr
    );
    // Get existing organizers from Events Calendar and create sorted array for dropdown list
    $organizers_unordered_arr = tribe_get_organizers();
    foreach ( $organizers_unordered_arr as $organizer ) {
        $organizers_arr[] = array(
            'ID'   => $organizer->ID,
            'Name' => $organizer->post_title,
        );
    }
    unset( $name_arr );
    unset( $name_lowercase_arr );
    $name_arr = array_column( $organizers_arr, 'Name' );
    array_multisort(
        $name_arr,
        SORT_ASC,
        SORT_LOCALE_STRING,
        $organizers_arr
    );
    $fields_arr = array(
        'ID'                   => __( 'ID', 'events-optimizer' ),
        'post_title'           => __( 'Name', 'events-optimizer' ),
        'post_content'         => __( 'Description', 'events-optimizer' ),
        '_EventStartDate'      => __( 'Start', 'events-optimizer' ),
        '_EventEndDate'        => __( 'End', 'events-optimizer' ),
        '_EventVenueID'        => __( 'Venue', 'events-optimizer' ),
        '_EventOrganizerID'    => __( 'Organizer', 'events-optimizer' ),
        '_EventURL'            => __( 'Website', 'events-optimizer' ),
        '_EventCost'           => __( 'Cost', 'events-optimizer' ),
        '_EventCurrencySymbol' => __( 'Currency Symbol', 'events-optimizer' ),
    );
    // Get the next index of the venue if function is called by "Show more" button
    
    if ( isset( $_POST['next_result'] ) ) {
        $next_result = urldecode( $_POST['next_result'] );
    } else {
        $next_result = 0;
    }
    
    $k = $next_result;
    // Set index to the next upcoming venue
    $i = 0;
    // Counter for maximum results per page
    $events_offset = array_count_key( $ignored_and_added_objects, 'tribe_events' );
    $event = tribe_get_events( [
        'posts_per_page' => $next_result + $events_offset + 3 * $max_results_per_page,
    ] );
    $max_events = wp_count_posts( 'tribe_events' )->publish;
    // Check if not all venues have been shown yet and maximum results has not been reached
    while ( $k < $max_events and $i < $max_results_per_page ) {
        // Hide venues that have no errors
        $found_error = false;
        $event_details = get_post_meta( $event[$k]->ID );
        $event_details = $event_details + array(
            'ID' => array( $event[$k]->ID ),
        );
        $event_details = $event_details + array(
            'post_title' => array( $event[$k]->post_title ),
        );
        $event_details = $event_details + array(
            'post_content' => array( $event[$k]->post_content ),
        );
        $event_details = $event_details + array(
            '_EventVenueID' => array( $event[$k]->_EventVenueID ),
        );
        $event_details = $event_details + array(
            '_EventOrganizerID' => array( $event[$k]->_EventOrganizerID ),
        );
        // If event isn't part of the ignored or added objects list show it
        
        if ( array_search( $event[$k]->ID, array_column( $ignored_and_added_objects, 'post_id' ) ) === false or $show_ignored_objects === 'true' or $show_added_objects === 'true' ) {
            echo  '<form action="" method="POST" id="save_event_wid_form_' . $k . '">' ;
            echo  '<br>' ;
            echo  '<table class="widefat fixed striped" ><thead><tr><th>' ;
            \esc_html_e( 'Field', 'events-optimizer' );
            echo  '</th><th>' ;
            \esc_html_e( 'Value', 'events-optimizer' );
            echo  '</th><th>' ;
            \esc_html_e( 'Status', 'events-optimizer' );
            echo  '</th></tr></thead><tbody>' ;
            foreach ( $fields_arr as $field => $fieldname ) {
                
                if ( isset( $event_details[$field][0] ) ) {
                    $value = $event_details[$field][0];
                } else {
                    $value = '';
                }
                
                switch ( $field ) {
                    case 'ID':
                        echo  '<tr><td>' . $fieldname . '</td><td><input type="hidden" name="' . $field . '" id="' . $field . '" value="' . $value . '"/><a href="/wp-admin/post.php?post=' . $event[$k]->ID . '&action=edit" target="_blank">' . $value . ' <img src="' . plugin_dir_url( __FILE__ ) . 'assets/images/external-link.png" class="external-link-icon" alt="External Link"/></a></td><td>--</td></tr>' ;
                        break;
                    case 'post_title':
                        
                        if ( empty($value) ) {
                            echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/> <a href="https://www.google.de/search?q=' . urlencode( $event_details['post_title'][0] ) . '" target="_blank"> <img src="' . plugin_dir_url( __FILE__ ) . 'assets/images/search.svg" class="search-icon" alt="Search"/></a></td><td><span class="field-status-error">' . $valid . '</td></tr>' ;
                        } else {
                            echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/> <a href="https://www.google.de/search?q=' . urlencode( $event_details['post_title'][0] ) . '" target="_blank"> <img src="' . plugin_dir_url( __FILE__ ) . 'assets/images/search.svg" class="search-icon" alt="Search"/></a></td><td><span class="field-status-ok">' . __( 'OK', 'events-optimizer' ) . '</span></td></tr>' ;
                        }
                        
                        break;
                    case 'post_content':
                        
                        if ( empty($value) ) {
                            echo  '<tr><td>' . $fieldname . '</td><td><textarea name="' . $field . '" id="' . $field . '" rows="8" cols="30"></textarea></td><td><span class="field-status-error">' . __( 'Field is empty', 'events-optimizer' ) . '</span></td></tr>' ;
                            if ( $ignored_field_event_description === 'false' ) {
                                $found_error = true;
                            }
                        } else {
                            echo  '<tr><td>' . $fieldname . '</td><td><textarea name="' . $field . '" id="' . $field . '" rows="8" cols="30">' . esc_textarea( $value ) . '</textarea></td><td><span class="field-status-ok">' . __( 'OK', 'events-optimizer' ) . '</span></td></tr>' ;
                        }
                        
                        break;
                    case '_EventStartDate':
                    case '_EventEndDate':
                        if ( $field === '_EventEndDate' and $value === '' ) {
                            $value = $event_details['_EventStartDate'][0];
                        }
                        
                        if ( $field === '_EventEndDate' and $value === $event_details['_EventStartDate'][0] ) {
                            echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . date_format( date_create( $value ), "Y-m-d H:i" ) . '" class="flexible-textfield"/></td><td><span class="field-status-warning">' . __( 'End is equal with start. Please check!', 'events-optimizer' ) . '</span></td></tr>' ;
                            break;
                        }
                        
                        
                        if ( empty($value) ) {
                            echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . date_format( date_create( $value ), "Y-m-d H:i" ) . '" class="flexible-textfield"/></td><td><span class="field-status-error">' . __( 'Field is empty', 'events-optimizer' ) . '</span></td></tr>' ;
                            if ( $field === '_EventStartDate' and $ignored_field_event_start === 'false' ) {
                                $found_error = true;
                            }
                            if ( $field === '_EventEndDate' and $ignored_field_event_end === 'false' ) {
                                $found_error = true;
                            }
                        } else {
                            echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . date_format( date_create( $value ), "Y-m-d H:i" ) . '" class="flexible-textfield"/></td><td><span class="field-status-ok">' . __( 'OK', 'events-optimizer' ) . '</span></td></tr>' ;
                        }
                        
                        break;
                    case '_EventVenueID':
                        $venues_list = '<select name="_EventVenueID">';
                        $venue_found = false;
                        foreach ( $venues_arr as $venue ) {
                            
                            if ( intval( $value ) === intval( $venue['ID'] ) ) {
                                $venues_list .= '<option value="' . $venue['ID'] . '" selected>' . $venue['Name'] . '</option>';
                                $venue_found = true;
                            } else {
                                $venues_list .= '<option value="' . $venue['ID'] . '">' . $venue['Name'] . '</option>';
                            }
                        
                        }
                        
                        if ( $venue_found ) {
                            $venues_list .= '</select>';
                            echo  '<tr><td>' . $fieldname . '</td><td>' . $venues_list . '</td><td><span class="field-status-ok">' . __( 'OK', 'events-optimizer' ) . '</span></td></tr>' ;
                        } else {
                            $venues_list .= '<option value="" selected>--</option>';
                            $venues_list .= '</select>';
                            echo  '<tr><td>' . $fieldname . '</td><td>' . $venues_list . '</td><td><span class="field-status-error">' . __( 'Field is empty', 'events-optimizer' ) . '</span></td></tr>' ;
                            if ( $ignored_field_event_venue === 'false' ) {
                                $found_error = true;
                            }
                        }
                        
                        break;
                    case '_EventOrganizerID':
                        $organizers_list = '<select name="_EventOrganizerID">';
                        $organizer_found = false;
                        foreach ( $organizers_arr as $organizer ) {
                            
                            if ( intval( $value ) === intval( $organizer['ID'] ) ) {
                                $organizers_list .= '<option value="' . $organizer['ID'] . '" selected>' . $organizer['Name'] . '</option>';
                                $organizer_found = true;
                            } else {
                                $organizers_list .= '<option value="' . $organizer['ID'] . '">' . $organizer['Name'] . '</option>';
                            }
                        
                        }
                        
                        if ( $organizer_found ) {
                            $organizers_list .= '</select>';
                            echo  '<tr><td>' . $fieldname . '</td><td>' . $organizers_list . '</td><td><span class="field-status-ok">' . __( 'OK', 'events-optimizer' ) . '</span></td></tr>' ;
                        } else {
                            $organizers_list .= '<option value="" selected>--</option>';
                            $organizers_list .= '</select>';
                            echo  '<tr><td>' . $fieldname . '</td><td>' . $organizers_list . '</td><td><span class="field-status-error">' . __( 'Field is empty', 'events-optimizer' ) . '</span></td></tr>' ;
                            if ( $ignored_field_event_organizer === 'false' ) {
                                $found_error = true;
                            }
                        }
                        
                        break;
                    case '_EventURL':
                        $valid = url_valid( $value );
                        
                        if ( $valid != '' ) {
                            echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/> <a href="https://www.google.de/search?q=' . urlencode( $event_details['post_title'][0] ) . '" target="_blank"> <img src="' . plugin_dir_url( __FILE__ ) . 'assets/images/search.svg" class="search-icon" alt="Search"/></a></td><td><span class="field-status-error">' . $valid . '</td></tr>' ;
                            if ( $ignored_field_event_website === 'false' ) {
                                $found_error = true;
                            }
                        } else {
                            echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/> <a href="' . $value . '" target="_blank"><img src="' . plugin_dir_url( __FILE__ ) . 'assets/images/external-link.png" class="external-link-icon" alt="External Link"/></a></td><td><span class="field-status-ok">' . __( 'OK', 'events-optimizer' ) . '</span></td></tr>' ;
                        }
                        
                        break;
                        // Check if currency symbol is in custom currency symbol list
                    // Check if currency symbol is in custom currency symbol list
                    case '_EventCurrencySymbol':
                        
                        if ( empty($value) ) {
                            echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/></td><td><span class="field-status-error">' . __( 'Field is empty', 'events-optimizer' ) . '</span></td></tr>' ;
                            if ( $ignored_field_event_currency_symbol === 'false' ) {
                                $found_error = true;
                            }
                        } else {
                            // Custom currency symbols available?
                            
                            if ( !empty($currency_symbols_arr) ) {
                                
                                if ( !in_array( $value, $currency_symbols_arr ) ) {
                                    echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/></td><td><span class="field-status-error">' . __( 'Currency symbol is not in custom currency symbol list', 'events-optimizer' ) . '</span></td></tr>' ;
                                    if ( $ignored_field_event_currency_symbol === 'false' ) {
                                        $found_error = true;
                                    }
                                } else {
                                    echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/></td><td><span class="field-status-ok">' . __( 'OK', 'events-optimizer' ) . '</span></td></tr>' ;
                                }
                            
                            } else {
                                echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/></td><td><span class="field-status-ok">' . __( 'OK', 'events-optimizer' ) . '</span></td></tr>' ;
                            }
                        
                        }
                        
                        break;
                    case '_EventCost':
                        
                        if ( !is_numeric( $value ) ) {
                            // set the local language for the date from the wordpress settings
                            echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/> <a href="https://www.google.de/search?q=' . urlencode( $event_details['post_title'][0] . ' ' . tribe_get_venue( $event[$k]->ID ) . ' ' . date_i18n( get_option( 'date_format' ), strtotime( $event_details['_EventStartDate'][0] ) ) . ' ' . __( 'Cost', 'events-optimizer' ) ) . '" target="_blank"> <img src="' . plugin_dir_url( __FILE__ ) . 'assets/images/search.svg" class="search-icon" alt="Search"/></a></td><td><span class="field-status-error">' . __( 'Field contains invalid characters or is empty', 'events-optimizer' ) . '</span></td></tr>' ;
                            if ( $ignored_field_event_cost === 'false' ) {
                                $found_error = true;
                            }
                        } else {
                            echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/></td><td><span class="field-status-ok">' . __( 'OK', 'events-optimizer' ) . '</span></td></tr>' ;
                        }
                        
                        break;
                        /*
                        					case '_thumbnail_id':
                        						if ( empty( $value )) {
                        							echo '<tr><td>'. $fieldname. '</td><td>-- <a href="/wp-admin/post.php?post='.$event[$k]->ID.'&action=edit" target="_blank"><img src="'.wp_get_attachment_image_url($value, 'thumbnail').'"><img src="'.plugin_dir_url( __FILE__ ).'assets/images/external-link.png" class="external-link-icon" alt="External Link"/></a></td><td><span class="field-status-error">'.__( 'Field is empty', 'events-optimizer' ).'</span></td></tr>';
                        							$found_error = true;						
                        						} else {					
                        							echo '<tr><td>'. $fieldname. '</td><td><a href="/wp-admin/post.php?post='.$event[$k]->ID.'&action=edit" target="_blank"><img src="'.wp_get_attachment_image_url($value, 'thumbnail').'"> <img src="'.plugin_dir_url( __FILE__ ).'assets/images/external-link.png" class="external-link-icon" alt="External Link"/></a></td><td><span class="field-status-ok">'.__( 'OK', 'events-optimizer' ).'</span></td></tr>';
                        						}
                        						break;
                        */
                    /*
                    					case '_thumbnail_id':
                    						if ( empty( $value )) {
                    							echo '<tr><td>'. $fieldname. '</td><td>-- <a href="/wp-admin/post.php?post='.$event[$k]->ID.'&action=edit" target="_blank"><img src="'.wp_get_attachment_image_url($value, 'thumbnail').'"><img src="'.plugin_dir_url( __FILE__ ).'assets/images/external-link.png" class="external-link-icon" alt="External Link"/></a></td><td><span class="field-status-error">'.__( 'Field is empty', 'events-optimizer' ).'</span></td></tr>';
                    							$found_error = true;						
                    						} else {					
                    							echo '<tr><td>'. $fieldname. '</td><td><a href="/wp-admin/post.php?post='.$event[$k]->ID.'&action=edit" target="_blank"><img src="'.wp_get_attachment_image_url($value, 'thumbnail').'"> <img src="'.plugin_dir_url( __FILE__ ).'assets/images/external-link.png" class="external-link-icon" alt="External Link"/></a></td><td><span class="field-status-ok">'.__( 'OK', 'events-optimizer' ).'</span></td></tr>';
                    						}
                    						break;
                    */
                    default:
                        
                        if ( empty($value) ) {
                            echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/></td><td><span class="field-status-error">' . __( 'Field is empty', 'events-optimizer' ) . '</span></td></tr>' ;
                            if ( $field === 'post_title' ) {
                                $found_error = true;
                            }
                        } else {
                            
                            if ( strlen( $value ) <= 4 ) {
                                echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/></td><td><span class="field-status-warning">' . __( 'Field content is very short', 'events-optimizer' ) . '</span></td></tr>' ;
                                if ( $field === 'post_title' ) {
                                    $found_error = true;
                                }
                            } else {
                                echo  '<tr><td>' . $fieldname . '</td><td><input type="text" name="' . $field . '" id="' . $field . '" value="' . $value . '" class="flexible-textfield"/></td><td><span class="field-status-ok">' . __( 'OK', 'events-optimizer' ) . '</span></td></tr>' ;
                            }
                        
                        }
                
                }
                // end switch
            }
            // end foreach
            echo  '</tbody>
				</table>	
				<p><input type="button" name="action" class="button button-primary" id="save_event_wid_button_' . $k . '" value="' . __( 'Save changes', 'events-optimizer' ) . '" onclick="save_form(' . "'" . $k . "'" . ');" />
				&nbsp;&nbsp;&nbsp;<input type="button" name="action" class="button delete" id="ignore_object_wid_button_' . $k . '" value="' . __( 'Ignore', 'events-optimizer' ) . '" onclick="ignore_object(' . "'" . $k . "'" . ');"' ;
            echo  "disabled" ;
            echo  '/>
				</p><div id="status_save_event_wid_' . $k . '">' . __( 'SAVING', 'events-optimizer' ) . '</div></p><br>
				</form>' ;
        }
        
        // end if array_search(
        $nonce = wp_create_nonce( 'events-optimizer-nonce' );
        ?>	

		<script  type='text/javascript'>
	
			function save_form(form_id){
				var datastring = jQuery("#save_event_wid_form_"+form_id).serialize(); 
				jQuery("#save_event_wid_button_"+form_id).attr("disabled", true); // Disable button while saving
				jQuery.ajax({
					type: "post",
					url: "admin-ajax.php",
					data: { 
						action: 'save_eve',
						_ajax_nonce: '<?php 
        echo  $nonce ;
        ?>' ,
						data: datastring,
					},
					beforeSend: function() {
						jQuery("#status_save_event_wid_"+form_id).fadeIn('fast');  // Fade in status when button is clicked
					},
					success: function(html) {
						jQuery("#status_save_event_wid_"+form_id).fadeOut('slow');	// Fade out status after data is saved
						jQuery("#save_event_wid_button_"+form_id).attr("disabled", false); // Enable button after saving
						jQuery("#save_event_wid_form_"+form_id).slideUp(); // Slide organizer form away after saving									
					}
				}); //close jQuery.ajax
				return false;	
			}
			function ignore_object(form_id){
				var datastring = jQuery("#save_event_wid_form_"+form_id).serialize() + '&type=tribe_events&status=ignored';
				jQuery("#ignore_object_wid_button_"+form_id).attr("disabled", true); // Disable button while ignoring
				jQuery.ajax({
					type: "post",
					url: "admin-ajax.php",
					data: { 
						action: 'add_obj',
						_ajax_nonce: '<?php 
        echo  $nonce ;
        ?>' ,
						data: datastring,
					},
					beforeSend: function() {
					},
					success: function(html) {
						jQuery("#status_save_event_wid_"+form_id).fadeOut('slow');	// Fade out status after data is ignored
						jQuery("#ignore_object_wid_button_"+form_id).attr("disabled", false); // Enable button after ignoring
						jQuery("#save_event_wid_form_"+form_id).slideUp(); // Slide venue form away after ignoring							
					}
				}); //close jQuery.ajax
				return false;	
			}						
		</script>	

		<style type='text/css'>
			#status_save_event_wid_<?php 
        echo  $k ;
        ?> { 
				display: none;
				clear: both;
				background: url(<?php 
        echo  plugin_dir_url( __FILE__ ) . 'assets/images/loading.gif' ;
        ?>) left top no-repeat; 
				text-align: left;
				padding: 0px 0px 30px 25px; 
			}
			<?php 
        if ( $found_error === false ) {
            echo  "#save_event_wid_form_" . $k . ", #save_event_wid_button_" . $k . " { display: none; }" ;
        }
        ?>	
		</style>

		<?php 
        // if invalid data is found increase counter for maximum results per page
        if ( $found_error === true ) {
            $i++;
        }
        $k++;
    }
    // end while
    
    if ( $k < $max_events ) {
        echo  '<div id="additional_results_events_wid_' . $k . '"><p><strong>' . __( 'Additional results available.', 'events-optimizer' ) . '</strong></p></div>' ;
        echo  '<p><input type="submit" name="action" class="button button-primary" id="show_more_events_wid_button_' . $k . '" value="' . __( 'Show more', 'events-optimizer' ) . '" onclick="show_more(' . "'" . $k . "'" . ');" /></p>' ;
        echo  '<div id="show_more_events_wid"></div>' ;
        echo  '<div id="status_show_more_events_wid_' . $k . '">' . __( 'ANALYZING', 'events-optimizer' ) . '</div>' ;
    } else {
        echo  '<span class="field-status-ok">' . __( 'No (more) results found!', 'events-optimizer' ) . '</span>' ;
    }
    
    $nonce = wp_create_nonce( 'events-optimizer-nonce' );
    ?>	
		
	<script  type='text/javascript'>
	
		function show_more(form_id){
       		jQuery("#show_more_events_wid_button_"+form_id).fadeOut('slow'); // Hide button after  		
    		jQuery("#show_more_events_wid_button_"+form_id).attr("disabled", true); // Disable button while retrieving more results
    		jQuery.ajax({
				type: "post",
				url: "admin-ajax.php",
				data: { 
					action: 'events_wid',
					next_result: form_id,
					_ajax_nonce: '<?php 
    echo  $nonce ;
    ?>' 
				},
				beforeSend: function() {
					jQuery("#status_show_more_events_wid_"+form_id).fadeIn('fast');  // Fade in status when button is clicked
					jQuery("#additional_results_events_wid_"+form_id).fadeOut('slow');	// Fade out text "Additional results ..." after data is saved
				},
				success: function(html) {
					jQuery("#show_more_events_wid_button_"+form_id).attr("disabled", false); // Enable button after
					jQuery("#status_show_more_events_wid_"+form_id).fadeOut('slow');	// Fade out status after data is saved
					jQuery("#show_more_events_wid").append(html); // Add the additional results to page
				}
    		}); //close jQuery.ajax
    		return false;	
    	}	
	</script>

	<style type='text/css'>
		#status_show_more_events_wid_<?php 
    echo  $k ;
    ?> { 
			display: none;
			clear: both;
			background: url(<?php 
    echo  plugin_dir_url( __FILE__ ) . 'assets/images/loading.gif' ;
    ?>) left top no-repeat; 
			text-align: left;
			padding: 0px 0px 30px 25px; 
		}
	</style>

	<?php 
    die;
}

//----------------------------------------------------------------------------------------
// Save event data (for events with invalid data)
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_save_eve', 'save_event' );
function save_event()
{
    check_ajax_referer( 'events-optimizer-nonce' );
    parse_str( $_POST['data'], $event );
    // Title and description can't be updated directly with the TEC API (empty fields gets ignored - bug?) and therefore it needs to be updated seperatly
    $event_post_arr = array(
        'ID'           => $event['ID'],
        'post_title'   => sanitize_text_field( $event['post_title'] ),
        'post_content' => wp_kses_post( $event['post_content'] ),
    );
    wp_update_post( $event_post_arr );
    // Update event with additional values
    $event_arr = array(
        'EventStartDate'      => substr( $event['_EventStartDate'], 0, 10 ),
        'EventEndDate'        => substr( $event['_EventEndDate'], 0, 10 ),
        'EventStartHour'      => substr( $event['_EventStartDate'], 11, 2 ),
        'EventEndHour'        => substr( $event['_EventEndDate'], 11, 2 ),
        'EventStartMinute'    => substr( $event['_EventStartDate'], 14, 2 ),
        'EventEndMinute'      => substr( $event['_EventEndDate'], 14, 2 ),
        'Venue'               => array(
        'VenueID' => sanitize_text_field( $event['_EventVenueID'] ),
    ),
        'Organizer'           => array(
        'OrganizerID' => sanitize_text_field( $event['_EventOrganizerID'] ),
    ),
        'EventURL'            => sanitize_text_field( $event['_EventURL'] ),
        'EventCost'           => sanitize_text_field( $event['_EventCost'] ),
        'EventCurrencySymbol' => sanitize_text_field( $event['_EventCurrencySymbol'] ),
    );
    //	'FeaturedImage'			=> sanitize_text_field( $event[ '_thumbnail_id' ] )
    tribe_update_event( $event['ID'], $event_arr );
    die;
}

//----------------------------------------------------------------------------------------
// Add button to genereate description for events, venues and organizers via hook
//----------------------------------------------------------------------------------------
//add_filter( 'edit_form_after_editor', 'add_button_to_generate_description');
//add_filter('media_buttons_context', 'add_button_to_generate_description');
//add_filter( 'edit_form_after_title', 'add_button_to_generate_description');
//add_action( 'media_buttons', 'add_button_to_generate_description');
//add_action( 'tribe_events_venue_before_metabox', 'add_button_to_generate_description');
//add_action( 'edit_form_before_editor', 'add_button_to_generate_description');
add_action( 'edit_form_after_editor', 'add_button_to_generate_description' );
function add_button_to_generate_description()
{
    $screen = get_current_screen();
    echo  "<br>" ;
    switch ( get_current_screen()->post_type ) {
        case 'tribe_events':
            generate_event_description();
            break;
        case 'tribe_venue':
            generate_venue_description();
            break;
        case 'tribe_organizer':
            generate_organizer_description();
            break;
    }
}

//----------------------------------------------------------------------------------------
// Generate description for single event
//----------------------------------------------------------------------------------------
function generate_event_description()
{
    // Activate only for Pro version
    
    if ( teo_fs()->can_use_premium_code__premium_only() and get_option( 'teo_openai_hide_generate_description_buttons' ) === 'false' ) {
        $nonce = wp_create_nonce( 'events-optimizer-nonce' );
        wp_enqueue_style( 'style.css', plugin_dir_url( __FILE__ ) . 'assets/css/style.css' );
        // Add button to existing event form
        ?>	
	
		<input type="button" name="action" id="generate_event_description_button" class="button-primary" value="<?php 
        \esc_attr_e( 'Generate description with OpenAI', 'events-optimizer' );
        ?>" />
		<span class="field-tip">
			&nbsp;<span class="question-mark-centered"><img src ="<?php 
        echo  plugin_dir_url( __FILE__ ) . 'assets/images/question-mark.png' ;
        ?>"></span>
			<span class="tip-content"><?php 
        echo  __( 'Event name, date, time and venue are used to generate the description. Please enter at least the name!', 'events-optimizer' ) ;
        ?></span>
		</span>

		<script type='text/javascript'>

			jQuery(document).ready(function() {

				jQuery("#generate_event_description_button").on('click', function()  {

					jQuery("#generate_event_description_button").attr("disabled", true); // Disable button while retrieving more results
					
					// Get existing content from TinyMCE editor or text editor
					if (typeof tinyMCE !== "undefined" && tinyMCE.activeEditor != null && !tinyMCE.activeEditor.isHidden()) {
 	
      					existing_content = tinymce.activeEditor.getContent();
					} else if ( jQuery('#content').length > 0 ) { 
						existing_content = jQuery('#content').val();
					}
					
					searchstring = 	document.getElementById("title").value + " " +	
									document.getElementById("EventStartDate").value + " " +
									document.getElementById("EventStartTime").value + " " +
									document.getElementById("select2-saved_tribe_venue-container").childNodes[1].textContent.trim();

					if ( searchstring.trim() === "" || document.getElementById("title").value === "" ) { 
    					alert( 'Event name, date, time and venue are used to generate the description. Please fill at least the name!', 'events_optimizer' );
   						jQuery("#generate_event_description_button").attr("disabled", false);// Enable button after	
					} else {							
					
						jQuery.ajax({
							type: "POST",
							url: "admin-ajax.php",
							data: { 		
								action: 'generate_event_description_with_oai', // call function "generate_event_description_with_oai"					
								_ajax_nonce: '<?php 
        echo  $nonce ;
        ?>',
								searchstring: searchstring,
								existing_content: existing_content
							},
							beforeSend: function() {
								jQuery("body").css("cursor", "progress");						
							},
							success : function( data ) {

								info = JSON.parse( data );
								
								// Error handling
								if ( info.error ) {
									alert( info.error );
								} else {

									// Check if info.description is defined and not empty
									if (info.description && info.description.trim() !== "") {
										// Add generated description to content field (either TinymCE or text editor)
										if (typeof tinyMCE !== "undefined" && tinyMCE.activeEditor != null && !tinyMCE.activeEditor.isHidden()) {
											tinyMCE.activeEditor.setContent(info.description);
										} else if (jQuery("#content").length > 0) {
											jQuery("#content").val(info.description);
										} else {
											console.error("Unable to set content: No suitable editor or text area found");
										}
									} else {
										console.warn("Description is empty or undefined");
									}
								}
								
								jQuery("#generate_event_description_button").attr("disabled", false); // Enable button after
								jQuery("body").css("cursor", "default");
							}
						})
					}
					return false;					
				});											
			});
			
		</script>
	
		<?php 
    }

}

//----------------------------------------------------------------------------------------
// Generate event description and hand it over to Java Script function above
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_generate_event_description_with_oai', 'generate_event_description_with_open_ai' );
function generate_event_description_with_open_ai()
{
}

//----------------------------------------------------------------------------------------
// Search events from Google data
// Creates a task with the results at DataForSEO
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_search_events_wgd', 'search_events_with_google_data' );
function search_events_with_google_data()
{
}

//----------------------------------------------------------------------------------------
// Show search results and add events from DataForSEO (Google events data)
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_add_events_wgd', 'add_events_with_google_data' );
function add_events_with_google_data()
{
}

//----------------------------------------------------------------------------------------
// Add event with Google Events
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_add_eve', 'add_event' );
function add_event()
{
}

//----------------------------------------------------------------------------------------
// Add object to list of added and ignored objects in WordPress options
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_add_obj', 'add_object' );
function add_object()
{
}

//----------------------------------------------------------------------------------------
// Remove object from list of added and ignored objects
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_remove_obj', 'remove_object' );
function remove_object()
{
}

//----------------------------------------------------------------------------------------
// Remove object(s) from list of added and ignored objects when according post is deleted
//----------------------------------------------------------------------------------------
add_action( 'wp_trash_post', 'wpdocs_trash_multiple_posts' );
function wpdocs_trash_multiple_posts( $post_id = '' )
{
}

// Removes object
function remove_object_when_post_deleted( $post_id )
{
}

//----------------------------------------------------------------------------------------
// Reset list of added or ignored objects
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_reset_object_li', 'reset_object_list' );
function reset_object_list()
{
}

//----------------------------------------------------------------------------------------
// Add images from Google images with DataForSEO
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_add_img', 'add_images' );
function add_images()
{
}

//----------------------------------------------------------------------------------------
// Add images from Google images in Featured Image meta box
//----------------------------------------------------------------------------------------
/*
function add_custom_button_to_featured_image_meta_box( $content, $post ) {
    //$value = get_post_meta( $post->ID, '_custom_meta_box_value', true );
    //$button_html = '<button>' . esc_html( $value ) . '</button>';

    $html = '<input type="text" name="teo_keyword" id="teo_keyword">';
	$html.= '<input type="submit" name="teo_search_images" id="teo_search_images" value="Search" class="button">';

	$content .= '<br>' . $html;
    return $content;
}
add_filter( 'admin_post_thumbnail_html', 'add_custom_button_to_featured_image_meta_box', 10, 2 );
*/
//----------------------------------------------------------------------------------------
// Save settings
//----------------------------------------------------------------------------------------
add_action( 'wp_ajax_save_set', 'save_settings' );
function save_settings()
{
    check_ajax_referer( 'events-optimizer-nonce' );
    $max_results_per_page = urldecode( $_POST['max_results_per_page'] );
    if ( $max_results_per_page > 20 ) {
        $max_results_per_page = 20;
    }
    update_option( 'teo_max_results_per_page', $max_results_per_page );
    die;
}

//----------------------------------------------------------------------------------------
// Google Places API: Place text search
// https://medium.com/@pauljkbs/simple-way-to-get-started-with-the-google-places-api-and-php-9a1f55f8919e
//----------------------------------------------------------------------------------------
function google_places_text_search( $query = '' )
{
}

//----------------------------------------------------------------------------------------
// Google Places API: Place details with place_id
//----------------------------------------------------------------------------------------
function get_place_details( $place_id = '' )
{
}

//----------------------------------------------------------------------------------------
// Google Places API: Place details with cid
// Possible Enhancement:
// https://developers.google.com/maps/documentation/places/web-service/place-id#save-id
//----------------------------------------------------------------------------------------
function get_place_details_cid( $cid = '' )
{
}

//----------------------------------------------------------------------------------------
// OpenAI API: Bulk generate a text description for venues or organizers
//----------------------------------------------------------------------------------------
/*
add_filter( 'bulk_actions-edit-tribe_events', 'bulk_generate_event_description' );

function bulk_generate_event_description( $bulk_actions ) {
  $bulk_actions['generate_event_description'] = __( 'Generate description', 'events-optimizer' );
  return $bulk_actions;
}

add_filter( 'bulk_actions-edit-tribe_venue', 'bulk_generate_venue_description' );

function bulk_generate_venue_description( $bulk_actions ) {
  $bulk_actions['generate_venue_description'] = __( 'Generate description', 'events-optimizer' );
  return $bulk_actions;
}

add_filter( 'bulk_actions-edit-tribe_organizer', 'bulk_generate_organizer_description' );

function bulk_generate_organizer_description( $bulk_actions ) {
  $bulk_actions['generate_organizer_description'] = __( 'Generate description', 'events-optimizer' );
  return $bulk_actions;
}

add_filter( 'handle_bulk_actions-edit-tribe_events', 'handle_generate_description_bulk_action', 10, 3 );
add_filter( 'handle_bulk_actions-edit-tribe_venue', 'handle_generate_description_bulk_action', 10, 3 );
add_filter( 'handle_bulk_actions-edit-tribe_organizer', 'handle_generate_description_bulk_action', 10, 3 );

function handle_generate_description_bulk_action( $redirect_to, $doaction, $post_ids ) {

	// Activate only for Pro version
	if ( teo_fs()->can_use_premium_code__premium_only() ) {
	
		if ( $doaction !== 'generate_event_description' && $doaction !== 'generate_venue_description' && $doaction !== 'generate_organizer_description') {
			return $redirect_to;
		}

		$language = get_option( 'teo_openai_language', 'English' );
		
		foreach ( $post_ids as $post_id ) {
	
			$post = get_post($post_id);
			if (empty($post->post_content)) {
				$existing_content = '';
			} else {
				$existing_content = $post->post_content;
			}

			switch( get_post_type($post_id) ) {
				case 'tribe_events':
					// $prompt = "Create a descriptive and informative event description in ".$language." for '".get_post_field( 'post_title', $post_id )."'. The description should be easy to read and include only factual information. Additionally, the description should be optimized for search engines. Provide a brief overview of the location's surroundings and nearby attractions. Include information on transportation options, such as nearby subway or bus stations. Mention any notable features or amenities of the location, such as a unique architecture or ample parking.";
					$prompt = "Write a detailed description in ".$language." for the event ".get_post_field( 'post_title', $post_id ).". If available describe what is special about it. If available descripe the interior. Avoid any repetitions. The description should be include only factual information. Use a professional tone.";

					break;
				case 'tribe_venue':
					//$prompt = "Create a descriptive and informative description in ".$language." for the venue '".get_post_field( 'post_title', $post_id )."'. The description should be easy to read and include only factual information. Additionally, the description should be optimized for search engines. Provide a brief overview of the surroundings and nearby attractions of the location. Include information on transportation options, such as nearby subway or bus stations. Mention any notable features or amenities of the location, such as a unique architecture or ample parking. Questions: What type of event will be taking place at this location? Are there any notable features or history of the location that should be mentioned in the description? Is there any parking or transportation information that is specific to the location that should be included in the description? Are there any frequently asked questions about the event location that should be included in the FAQ section?";
					$prompt = "Write a detailed description in ".$language." for the venue ".get_post_field( 'post_title', $post_id ).". If available describe what is special about it. If available descripe the interior. Avoid any repetitions. The description should be include only factual information. Use a professional tone.";

					break;
				case 'tribe_organizer':
					//$prompt = "Create a descriptive and informative description in ".$language." for the event organizer '".get_post_field( 'post_title', $post_id )."'. The description should be easy to read and include only factual information. Additionally, the description should be optimized for search engines. Questions: Are there any notable features or history of the organizer that should be mentioned in the description? Are there any frequently asked questions about the event organizer that should be included in the FAQ section?";
					$prompt = "Write a detailed description in ".$language." for the event organizer ".get_post_field( 'post_title', $post_id ).". If available describe what is special about it. If available descripe the interior. Avoid any repetitions. The description should be include only factual information. Use a professional tone.";

					break;
			}


			$description = generate_description( $prompt );
			if ( isset( $description[ 'error' ] ) ) {
				$description = $description['error'];
			} else {
				$description = $description[ 'description' ];
			}

			if (get_option( 'teo_openai_replace_content') === 'false') {
				$description = $existing_content. $description;
			}

			$args = array(
			'ID' => $post_id,
			'post_content' => $description,
			);
			wp_update_post( $args );
		}
		$redirect_to = add_query_arg( 'generated_descriptions', count( $post_ids ), $redirect_to );
		return $redirect_to;
	}
}
*/
//----------------------------------------------------------------------------------------
// OpenAI API: Automatically generate a text description for a venue or an organizer
//----------------------------------------------------------------------------------------
function generate_description( $prompt = '' )
{
}

//----------------------------------------------------------------------------------------
// Check if URL is valid and reachable
//----------------------------------------------------------------------------------------
function url_valid( $url = true )
{
    // Check URLs with Google Safe Browsing
    $google_safe_browsing = get_option( 'teo_google_safe_browsing', 'false' );
    $validate_url_https = get_option( 'teo_validate_url_https', 'false' );
    $validate_url_redirected_moved = get_option( 'teo_validate_url_redirected_moved', 'false' );
    // If URL contains something check what's inside
    
    if ( !empty($url) ) {
        // Is URL syntax valid? Accept as ok, even when http:// or https:// or www is missing
        $url = filter_var( $url, FILTER_SANITIZE_URL );
        // Check on missing https / http
        
        if ( $validate_url_https === 'false' ) {
            $valid_urls = array( $url, 'http://' . $url, 'https://' . $url );
        } else {
            $valid_urls = array( $url, 'http://www' . $url, 'https://www' . $url );
        }
        
        foreach ( $valid_urls as $valid_url ) {
            // Check if URL syntax is valid
            
            if ( filter_var( $valid_url, FILTER_VALIDATE_URL ) ) {
                // Get the headers of the URL
                unset( $file_headers );
                $file_headers = @get_headers( $valid_url );
                // Check if URL is redirected or permanently moved
                if ( teo_fs()->can_use_premium_code__premium_only() and $validate_url_redirected_moved === 'true' ) {
                    if ( isset( $file_headers[0] ) ) {
                        if ( strpos( $file_headers[0], '301' ) > 0 or strpos( $file_headers[0], '302' ) > 0 ) {
                            return __( 'URL is redirected or moved permanently', 'events-optimizer' );
                        }
                    }
                }
                // Check if URL is reachable (status code <> 200)
                
                if ( isset( $file_headers[0] ) and (bool) preg_match( '/^HTTP\\/\\d\\.\\d\\s+2\\d{2}\\s+/', $file_headers[0] ) ) {
                    // Activate only for Pro version
                    
                    if ( teo_fs()->can_use_premium_code__premium_only() and $google_safe_browsing === 'true' ) {
                        // Check if URL is marked as suspicous by Google
                        $googleSafebrowsing = new GoogleSafebrowsing();
                        $status = $googleSafebrowsing->check( $valid_url );
                        
                        if ( $status === GoogleSafebrowsing::OK ) {
                            // URL is valid
                            return '';
                        } else {
                            return __( 'Google says this URL is suspicious! Please check!', 'events-optimizer' );
                        }
                    
                    } else {
                        // URL is valid
                        return '';
                    }
                
                } else {
                    return __( 'URL is not reachable', 'events-optimizer' );
                }
            
            } else {
                return __( 'URL syntax is not valid', 'events-optimizer' );
            }
        
        }
        // end foreach
        // URL is empty
    } else {
        return __( 'Field is empty', 'events-optimizer' );
    }

}

//----------------------------------------------------------------------------------------
// Check if URL is marked as suspicious by Google
// https://transparencyreport.google.com/safe-browsing/search?url
//----------------------------------------------------------------------------------------
class GoogleSafebrowsing
{
    const  OK = 0 ;
    const  SUSPICIOUS = 201 ;
    const  ENDPOINT = "https://www.google.com/safebrowsing/diagnostic?output=jsonp&site=" ;
    /**
     * @param string $url
     *
     * @return integer
     */
    public function check( $url )
    {
        if ( filter_var( $url, FILTER_VALIDATE_URL ) === false ) {
            throw new \Exception( "Invalid URL" );
        }
        $safebrowsing = @file_get_contents( self::ENDPOINT . $url );
        if ( strpos( $safebrowsing, '"listed"' ) ) {
            return self::SUSPICIOUS;
        }
        return self::OK;
    }

}
//----------------------------------------------------------------------------------------
// Check if mail address is valid
//----------------------------------------------------------------------------------------
function mail_valid( $mail = true )
{
    // Check if mail is empty
    if ( empty($mail) ) {
        return __( 'Mail is empty!', 'events-optimizer' );
    }
    // Check if mail syntax is valid
    if ( !filter_var( $mail, FILTER_VALIDATE_EMAIL ) ) {
        return __( 'Mail is not valid', 'events-optimizer' );
    }
    // Check if domain exists and is accepting emails
    /* Is not working reliable
    	list( , $domain) = explode('@', $email);
        if (! checkdnsrr( $domain, 'MX' ) ) {
            return __( 'Domain does not exist or accepting mails.', 'events-optimizer' );
        }
    	*/
}

//----------------------------------------------------------------------------------------
// Check if phone number is valid
// https://uibakery.io/regex-library/phone-number-php
//----------------------------------------------------------------------------------------
function phone_number_valid( $number = true )
{
    // Check if phone number is empty
    if ( empty($number) ) {
        return __( 'Phone number is empty', 'events-optimizer' );
    }
    // Define the regular expression pattern to match a valid international phone number format
    $pattern = '/^\\+?\\d{1,4}?[-.\\s]?\\(?\\d{1,3}?\\)?[-.\\s]?\\d{1,4}[-.\\s]?\\d{1,4}[-.\\s]?\\d{1,9}$/';
    // Check if the given phone number matches the pattern
    if ( preg_match( $pattern, $number ) != 1 ) {
        return __( 'Phone number is not valid', 'events-optimizer' );
    }
}

//----------------------------------------------------------------------------------------
// Create clean URL for SEO (e.g. for event image file name)
// https://stackoverflow.com/questions/4783802/converting-string-into-web-safe-uri
//----------------------------------------------------------------------------------------
function get_clean_url( $str, $replace = array(), $delimiter = '-' )
{
    setlocale( LC_ALL, 'en_US.UTF8' );
    if ( !empty($replace) ) {
        $str = str_replace( (array) $replace, ' ', $str );
    }
    $clean = preg_replace( array(
        '/Ä/',
        '/Ö/',
        '/Ü/',
        '/ä/',
        '/ö/',
        '/ü/'
    ), array(
        'Ae',
        'Oe',
        'Ue',
        'ae',
        'oe',
        'ue'
    ), $str );
    $clean = iconv( 'UTF-8', 'ASCII//TRANSLIT', $clean );
    $clean = preg_replace( "/[^a-zA-Z0-9\\/_|+ -]/", '', $clean );
    $clean = strtolower( trim( $clean, '-' ) );
    $clean = preg_replace( "/[\\/_|+ -]+/", $delimiter, $clean );
    return $clean;
}

//----------------------------------------------------------------------------------------
// Count elements in array with specific key
// https://stackoverflow.com/questions/11558397/count-specific-values-in-multidimensional-array/11558780
//----------------------------------------------------------------------------------------
function array_count_key( $array, $find )
{
    $i = 0;
    foreach ( $array as $key => $value ) {
        if ( $value['type'] === $find ) {
            $i++;
        }
    }
    return $i;
}
